/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dao;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.StatementProvider;
import java.io.Serializable;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

/**
 * @author grassi
 *
 */
public abstract class AbstractDAO implements Serializable, IFaceQueryProvider {
	private static final long serialVersionUID = 1632576031237433856L;
	protected ArrayList _parameterList = null ;
	protected DefaultRequestContext _requestContext ;
	protected SourceBean _configQuery ;
	protected DataConnection _dataConnection ;
	protected String _statementType ;
	protected Object _returnCode ;

	/**
	 * E' sempre il primo metodo richiamato dall'oggetto chiamante;
	 *   ha la responsabilit di inizializzare le variabili di istanza.
	 * @param statementProvider 
	 * @return String
	 */
	public final String getQuery(StatementProvider statementProvider) {
		_dataConnection = statementProvider.getDataConnection() ;
		_configQuery = statementProvider.getConfigQuery() ;
		_requestContext = statementProvider.getRequestContext() ;
		_statementType = _configQuery.getName() ;
		String ret = "" ;
		try {
			ret = getQuery() ;
		} catch (Exception ex) {
			TracerSingleton.log(Constants.QUERYTRACE, TracerSingleton.CRITICAL, "AbstractDAO::getQuery:", ex);
			String userMess = ex.getMessage() ;
			if (userMess != null && userMess.indexOf(".\r") > 0) 
				userMess = userMess.substring(0, userMess.indexOf(".\r")) ;
			else if (userMess != null && userMess.indexOf(".\n") > 0) 
					userMess = userMess.substring(0, userMess.indexOf(".\n")) ;
			_requestContext.getResponseContainer().getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, userMess));
		}
		
		return ret ;
	}

	/**
	 * Ritorna la lista con i parametri aggiuntivi
	 * @param configQuery
	 */
	public ArrayList getParameterList() {
		return _parameterList ;
	}

	protected abstract String getQuery() throws Exception ;

	protected abstract String getListQuery() ;
	protected abstract String getDeleteQuery() ;
	protected abstract String getInsertQuery() ;
	protected abstract String getUpdateQuery() ;
	protected abstract String getSelectQuery() ;
	
	protected abstract String[] getParametersForUpdateQuery() ;
	protected abstract String[] getParametersForSelectQuery() ;

	/**
	 * Ricerca e ritorna lo statement nella configurazione
	 * @param statementName
	 * @return
	 */
	protected String getStatementFromConfiguration(String statementName) {
        return SQLStatements.getStatement(statementName);
	}
	
	/**
	 * Dal nome del bean di configurazione ricerca e ritorna lo statement associato
	 * @param configQuery
	 * @return
	 */
	protected String getStatement() {
		String statementName = "" ;
		if (_statementType.equalsIgnoreCase("LIST_QUERY"))
			statementName = getListQuery() ;
		else if (_statementType.equalsIgnoreCase("DELETE_QUERY"))
			statementName = getDeleteQuery() ;
		else if (_statementType.equalsIgnoreCase("UPDATE_QUERY"))
			statementName = getUpdateQuery() ;
		else if (_statementType.equalsIgnoreCase("INSERT_QUERY"))
			statementName = getInsertQuery() ;
		else if (_statementType.equalsIgnoreCase("SELECT_QUERY"))
			statementName = getSelectQuery() ;
		
		return getStatementFromConfiguration(statementName) ;
	}

	/**
	 * Decodifica il tipo di azione a seconda dello statement invocato
	 * @param configQuery
	 * @return
	 */
	protected String getType() {
		String type = "" ;
		if (_statementType.equalsIgnoreCase("LIST_QUERY")
			|| _statementType.equalsIgnoreCase("SELECT_QUERY"))
			type = QueryExecutor.READ ;
		else if (_statementType.equalsIgnoreCase("DELETE_QUERY"))
			type = QueryExecutor.DELETE ;
		else if (_statementType.equalsIgnoreCase("UPDATE_QUERY"))
			type = QueryExecutor.UPDATE ;
		else if (_statementType.equalsIgnoreCase("INSERT_QUERY"))
			type = QueryExecutor.CREATE ;
		
		return type ;
	}
	
	/**
	 * Ritorna un'array list con i parametri richiesti per l'esecuzione dello statement
	 * 
	 * @param dataConnection
	 * @param requestContext
	 * @param parNames
	 * @return
	 */
	protected ArrayList getParametersValue(String[] parNames) {
		ArrayList parameterList = new ArrayList() ;
		for (int i=0 ; parNames != null && i < parNames.length ; i++) {
			Object value = _requestContext.getServiceRequest().getAttribute(parNames[i]) ;
			if (value!=null) {
				parameterList.add(_dataConnection.createDataField("", Types.VARCHAR, value)) ;
			}
			else {
				// Modifica inserita per il trattamento del blank
				char blank = 32;
				parameterList.add(_dataConnection.createDataField("", Types.VARCHAR, blank)) ;
			}
		}
		return parameterList ;
	}

	protected String getUserName() {
		return "USER" ;
	}
	protected String getProgramName() {
		return "MASTER" ;
	}
	protected String getProgramLibraryName() {
		return "*LIBL" ;
	}
	protected String getJobName() {
		return "WAS" ;
	}
	protected String getJobNumber() {
		return "999999" ;
	}
	protected String getCurrentDate() {
		return "20070330" ;
	}
	protected String getCurrentTime() {
		return "170000" ;
	}
	
	protected int getVersion() throws Exception {
		return 0;
	}
	
	protected void addParameter(DataField par) {
		if (_parameterList == null)
			_parameterList = new ArrayList() ;
		_parameterList.add(par) ;
	}

	public Object getReturnCode() {
		return _returnCode;
	}
}
