/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dbaccess;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.SessionContainer;
import it.eng.spago.dbaccess.factory.ConnectionPoolFactory;
import it.eng.spago.dbaccess.pool.ConnectionPoolInterface;
import it.eng.spago.dbaccess.pool.NativePoolWrapper;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.command.std.StoredProcedureCommand;
import it.eng.spago.dbaccess.sql.mappers.SQLMapper;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.security.IEngUserProfile;
import it.eng.spago.tracing.TracerSingleton;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.sql.DataSource;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
 * Questa Classe  il manager di tutti i pool di connessioni registrate con il
 * sottosistema di accesso ai dati e ha le seguenti responsabilit :
 * 
 * <li> Mantiene tutti i pool registrati con il sottositema di accesso ai dati
 * <li> Mette a disposizioni connessioni al DB sui diversi pool registrati nel
 * sottositema di accesso ai dati
 * <li> Mette a disposizione connessioni al DB su pool registrati in un contesto
 * JNDI
 * 
 * @author Andrea Zoppello
 * @version 1.0
 * 
 * Esempio di StoreProcedure creata su AS400 utilizzata per effettuare la modifica dinamica della lista
 * librerie
 * 
 *  CREATE PROCEDURE QGPL/J2EE_LIBL(IN libl VARCHAR (2750 ), IN   
 *  crtdft VARCHAR (10 )) LANGUAGE RPGLE NOT DETERMINISTIC NO SQL 
 *  EXTERNAL NAME QGPL/J2EE_LIBL PARAMETER STYLE GENERAL          
 * 
 * 
 */
public class DataConnectionManager {
	private static DataConnectionManager _instance = null;
	private ConnectionPoolInterface _defaultConnectionPool = null;
	private static HashMap _registeredConnectionPool = null;
	private HashMap _driversVersion = null;
	private HashMap _sqlMappers = null;
	private SQLMapper _defaultSQLMapper = null;
	private String _defaultDriverVersion = null;

	private DataConnectionManager() throws EMFInternalError {
		_registeredConnectionPool = new HashMap();
		_driversVersion = new HashMap();
		_sqlMappers = new HashMap();
		//
		// Retrive the names of all configured DataSources
		//
		List registeredPoolNames = Configurator.getInstance().getRegisteredConnectionPoolNames();
		Iterator it = registeredPoolNames.iterator();
		String connectionPoolName = null;
		ConnectionPoolDescriptor connectionPoolDescriptor = null;
		ConnectionPoolInterface connectionPool = null;
		//
		// Create First Connession Register it and Then set it as Default
		//
		if (it.hasNext()) {
			try {
				connectionPoolName = ((String) it.next());
				connectionPoolDescriptor = Configurator.getInstance().getConnectionPoolDescriptor(connectionPoolName);
				connectionPool = ConnectionPoolFactory.createConnectionPool(connectionPoolDescriptor);
				_registeredConnectionPool.put(connectionPoolDescriptor.getConnectionPoolName(), connectionPool);
				_defaultConnectionPool = connectionPool;
				_driversVersion.put(connectionPoolDescriptor.getConnectionPoolName(), connectionPoolDescriptor.getConnectionPoolParameter("driverVersion").getValue());
				_defaultDriverVersion = connectionPoolDescriptor.getConnectionPoolParameter("driverVersion").getValue();
				_defaultSQLMapper = (SQLMapper) ((Class.forName(connectionPoolDescriptor.getConnectionPoolParameter("sqlMapperClass").getValue())).newInstance());
				_sqlMappers.put(connectionPoolDescriptor.getConnectionPoolName(), _defaultSQLMapper);
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "DataConnectionManager::DataConnectionManager: registered connection pool [" + connectionPoolName + "] as default connection pool");
				SQLMapper tempMapper;
				while (it.hasNext()) {
					tempMapper = null;
					connectionPoolName = ((String) it.next());
					connectionPoolDescriptor = Configurator.getInstance().getConnectionPoolDescriptor(connectionPoolName);
					connectionPool = ConnectionPoolFactory.createConnectionPool(connectionPoolDescriptor);
					_registeredConnectionPool.put(connectionPoolDescriptor.getConnectionPoolName(), connectionPool);
					_driversVersion.put(connectionPoolDescriptor.getConnectionPoolName(), connectionPoolDescriptor.getConnectionPoolParameter("driverVersion").getValue());
					tempMapper = (SQLMapper) ((Class.forName(connectionPoolDescriptor.getConnectionPoolParameter("sqlMapperClass").getValue())).newInstance());
					_sqlMappers.put(connectionPoolDescriptor.getConnectionPoolName(), tempMapper);
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "DataConnectionManager::DataConnectionManager: registered connection pool data source [" + connectionPoolName + "] in DataConnectionManager");
				} // while (it.hasNext())
			} // try
			catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::DataConnectionManager:", ex);
				throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::DataConnectionManager: cannot initialize DataConnectionManager error occurred during the inizialization of "
						+ connectionPoolDescriptor.getConnectionPoolName(), ex);
			} // catch (Exception ex) try
		} // if (it.hasNext())
		else
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "DataConnectionManager::DataConnectionManager: "
					+ "ATTENTION NO CONNECION POOL REGISTERED WITH MANAGER PROBABLY YOU WILL USE THIS MANAGER ONLY AS AN INTERFACE TO APPLICATION SERVER");
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				release();
			} // public void run()
		} // new Thread()
				);
	} // private DataConnectionManager() throws EMFInternalError

	/**
	 * Questo metodo serve per ottenere l'unica istanza dell'oggetto
	 * DataConnectionManager
	 * 
	 * @return DataConnectionManager il manager di tutti i pool registrati
	 */
	public static DataConnectionManager getInstance() throws EMFInternalError {
		if (_instance == null) {
			synchronized (DataConnectionManager.class) {
				if (_instance == null)
					_instance = new DataConnectionManager();
			} // synchronized(DataConnectionManager.class)
		} // if (_instance == null)
		return _instance;
	} // public static DataConnectionManager getInstance()

	/**
	 * Metodo per ottenere una connessione dal pool di default (il primo nome
	 * registrato nel file xml)
	 * 
	 * @return un oggetto di tipo <B>DataConnection</B> rappresentante la
	 *         connessione al db
	 * @return un oggetto di tipo <B>DataConnection</B> rappresentante la
	 *         connessione al db
	 * @throws <B>EMFInternalError
	 *             </B> - Se si verifica qualche problema nel recuperare la
	 *             connessione al db
	 */
	public DataConnection getConnection() throws EMFInternalError {
		Monitor monitor = MonitorFactory.start("model.data-access.get-connection.default");
		try {
			Connection conn = _defaultConnectionPool.getConnection(null);
			if (conn != null)
				return new DataConnection(conn, _defaultDriverVersion, _defaultSQLMapper);
			else {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
				throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
			}
		} // try
		catch (SQLException ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:", ex);
			throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection: error in getting connection from the pool", ex);
		} // catch (SQLException ex) try
		finally {
			monitor.stop();
		} // finally
	} // public DataConnection getConnection() throws EMFInternalError

	/**
	 * Metodo per ottenere una connessione dal pool identificato dal parametro
	 * passato(il primo nome registrato nel file xml)
	 * 
	 * @param <B>String
	 *            </B> connectionPoolName - il nome del pool dal quale si vuole
	 *            ottenere una connessione
	 * @return un oggetto di tipo <B>DataConnection</B> rappresentante la
	 *         connessione al db
	 * @throws <B>EMFInternalError
	 *             </B> - Se si verifica qulache problema nel recuperare la
	 *             connessione al db
	 */
	public DataConnection getConnection(String connectionPoolName) throws EMFInternalError {
		return getConnection(connectionPoolName, null) ;
	}

	public DataConnection getConnection(String connectionPoolName, final RequestContainer requestContainer) throws EMFInternalError {
		Monitor monitor = MonitorFactory.start("model.data-access.get-connection." + connectionPoolName.toLowerCase());
		boolean error=false;
		try {
			/*
			 * System.err.println("GET CONNECTION("+connectionPoolName+") - from
			 * Thread:" + Thread.currentThread().getId()) ; // TODO: Andrea
			 */
			ConnectionPoolInterface cp = (ConnectionPoolInterface) _registeredConnectionPool.get(connectionPoolName);
			if (cp == null)
				throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection: cannot get connection pool " + connectionPoolName + " not present in DataConnectionManager");
			else {
				Connection conn = cp.getConnection(requestContainer);
				if (conn != null) {
					DataConnection dataConnection = new DataConnection(conn, (String) _driversVersion.get(connectionPoolName), (SQLMapper) _sqlMappers.get(connectionPoolName));
					
					ConnectionPoolParameter useOldMetadataBehaviour_parameter = (ConnectionPoolParameter)Configurator.getInstance().getConnectionPoolDescriptor(connectionPoolName).getConnectionPoolParameter("useOldMetadataBehaviour"); 
					ConnectionPoolParameter allowNullValues_parameter = (ConnectionPoolParameter)Configurator.getInstance().getConnectionPoolDescriptor(connectionPoolName).getConnectionPoolParameter("allowNullValues"); 
					
					if (useOldMetadataBehaviour_parameter!=null) {
						boolean useOldMetadataBehaviour = Boolean.parseBoolean(useOldMetadataBehaviour_parameter.getValue());
						dataConnection.setOldAliasMetadataBehavior(useOldMetadataBehaviour);
					}
					if (allowNullValues_parameter!=null) {
						boolean allowNullValues = Boolean.parseBoolean(allowNullValues_parameter.getValue());
						dataConnection.setAllowNullValues(allowNullValues);
					}

					//Setta il numero massimo di righe che un dataSet pu restituire
					ConnectionPoolParameter maxRows_parameter = (ConnectionPoolParameter)Configurator.getInstance().getConnectionPoolDescriptor(connectionPoolName).getConnectionPoolParameter("maxRows");
					
					if(maxRows_parameter!=null) {
						int maxRows = Integer.parseInt(maxRows_parameter.getValue());
						dataConnection.setMaxRows(maxRows);
					}
					
					/*
					 * System.err.println(" OPENED CONNECTION IN AUTOCOMMIT=" +
					 * dataConnection.getInternalConnection().getAutoCommit() + "-
					 * from Thread:" + Thread.currentThread().getId()) ; //
					 * TODO: Andrea
					 */
					/**
					 * Recupero la lista librerie associata all'applicazione e al ruolo ed al pool
					 * qualora non vi fosse nessuna lista librerie associata al profilo verr utilizzata
					 * quella definita a livello di datasource (vedi risorse jdbc in WAS) Gab
					 */
					String libraryList = null ;
					if(requestContainer!=null && connectionPoolName!=null) {
						SessionContainer permanentContainer = requestContainer.getSessionContainer().getPermanentContainer();
						IEngUserProfile ucx = (IEngUserProfile)permanentContainer.getAttribute(IEngUserProfile.ENG_USER_PROFILE) ;
						libraryList = ucx.getLibraryList(connectionPoolName);
					}

					if (libraryList != null && libraryList.length()>0) {
						error=dataConnection.changeLibraryList(libraryList);
					}
						
					if (error ){
						TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection: errore durante la modifica della lista librerie");
						throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection: errore durante la modifica della lista librerie");				
					}
					libraryList=null;
					
					/**
					 * Ritorna la connessione presa dal pool con l'eventuale lista librerie
					 */
					
					return dataConnection; 
					
				} else {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
					throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
				}
			} // if (cp == null) else
		} // try
		catch (SQLException ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:", ex);
			throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection: error in getting connection from the pool", ex);
		} // catch (SQLException ex) try
		finally {
			monitor.stop();
		} // finally
	} // public DataConnection getConnection(String connectionPoolName) throws
		// EMFInternalError

	/**
	 * Metodo per ottenere una connessione dal pool definito nel Contesto ctx
	 * con jndiName
	 * 
	 * @param <B>Context
	 *            </B> ctx - il contesto jndi da cui ricavare il pool
	 * @param <B>String
	 *            </B> jndiName - il nome jndi della connessione nel contesto
	 *            ctx
	 * @SQLMapper mapper - l'oggetto sqlMapper per la connessione
	 * @return un oggetto di tipo <B>DataConnection</B> rappresentante la
	 *         connessione al db
	 * @throws <B>EMFInternalError
	 *             </B> - Se si verifica qualche problema nel recuperare la
	 *             connessione al db
	 */
	public DataConnection getConnection(Context ctx, String jndiName, SQLMapper mapper) throws EMFInternalError {
		// This method allow a Client to use ConnectionPool Defined in
		// Application Server Context
		Monitor monitor = MonitorFactory.start("model.data-access.get-connection." + jndiName.toLowerCase());
		try {
			DataSource cpds = (DataSource) ctx.lookup(jndiName);
			if (cpds == null)
				throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection: cannot get connection pool " + jndiName + " not present in jndi context");
			else {
				NativePoolWrapper nativePool = new NativePoolWrapper(cpds);
				Connection conn = nativePool.getConnection();
				if (conn != null) {
					DataConnection dataConnection = new DataConnection(conn, String.valueOf(2.0), mapper);
					return dataConnection;
				} else {
					TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
					throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection:il pool ha ritornato una connessione null");
				}
			} // if (cpds == null) else
		} // try
		catch (NamingException ne) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:", ne);
			throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection:", ne);
		} // catch (NamingException ne)
		catch (SQLException sqle) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::getConnection:", sqle);
			throw new EMFInternalError(EMFErrorSeverity.ERROR, "DataConnectionManager::getConnection:", sqle);
		} // catch (SQLException sqle) try
		finally {
			monitor.stop();
		} // finally
	} // public DataConnection getConnection(Context ctx, String jndiName,
		// SQLMapper mapper) throws EMFInternalError

	public static void release() {
		if (_instance != null) {
			synchronized (DataConnectionManager.class) {
				if (_instance != null) {
					Collection registeredConnectionPools = _registeredConnectionPool.values();
					Iterator registeredConnectionPoolsIterator = registeredConnectionPools.iterator();
					while (registeredConnectionPoolsIterator.hasNext()) {
						ConnectionPoolInterface registeredConnectionPool = (ConnectionPoolInterface) registeredConnectionPoolsIterator.next();
						try {
							registeredConnectionPool.release();
						} // try
						catch (SQLException sqle) {
							TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "DataConnectionManager::release:", sqle);
						} // catch (SQLException sqle) try
					} // while (registeredConnectionPoolsIterator.hasNext())
					_instance = null;
				} // if (_instance != null)
			} // synchronized(DataConnectionManager.class)
		} // if (_instance != null)
	} // public synchronized void release()

} // public class DataConnectionManager