/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.dispatching.jmschannel;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.QueueSender;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class AdapterJMSProxy implements Serializable {
	private static final long serialVersionUID = 1L;
	private InitialContext iniCtx = null;
	private SourceBean _config = null;
	private ConnectionFactory _connectionFactory;
	private Destination _destinationQueue;

	/**
	 * This class send a message throw JMS channel
	 * 
	 * @param  queueConfigName  key for configuration queue in queues.xml
	 * @throws JMSException
	 * @throws NamingException
	 */
	public AdapterJMSProxy(String queueConfigName) throws JMSException, NamingException {
		_config = (SourceBean) ConfigSingleton.getInstance().getFilteredSourceBeanAttribute("QUEUES.QUEUE", "name", queueConfigName);
		if (_config == null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "AdapterJMSProxy: \"" + queueConfigName + "\" not found in configuration QUEUES.QUEUE");
			throw new NamingException("\"" + queueConfigName + "\" not found in configuration QUEUES.QUEUE");
		}
		create();
	}

	/**
	 * Retrive destination queue
	 * @throws NamingException
	 */
	private void create() throws NamingException {
		iniCtx = new InitialContext();
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION, "AdapterJMSProxy::create: connectionFactory=" + _config.getAttribute("connectionFactory") + " QueueName=" + _config.getAttribute("queueName"));
		_connectionFactory = (ConnectionFactory) iniCtx.lookup((String) _config.getAttribute("connectionFactory"));
		_destinationQueue = (Destination) iniCtx.lookup((String) _config.getAttribute("queueName"));
	}

	/**
	 * Send a message
	 * @param message
	 * @throws JMSException
	 * @throws NamingException
	 */
	public void sendMessage(String message) throws JMSException, NamingException {
		Session session = null;
		Connection connection = null;
		try {
			connection = _connectionFactory.createConnection();
			boolean isTransacted = Boolean.parseBoolean((String) _config.getAttribute("SESSION.transacted", "false"));
			int acknowledgeMode = Integer.parseInt((String) _config.getAttribute("SESSION.acknowledgeMode", "1"));
			session = connection.createSession(isTransacted, acknowledgeMode);

			sendMessageUsingMessageProducer(session, message);
		} finally {
			if (session != null)
				session.close();

			if (connection != null)
				connection.close();
		}
	}

	private void sendMessageUsingMessageProducer(Session session, String message) throws JMSException, NamingException {
		MessageProducer messageProducer = null;
		try {
			messageProducer = session.createProducer(_destinationQueue);
			int deliveryMode = Integer.parseInt((String) _config.getAttribute("MESSAGE_PRODUCER.DELIVERY_MODE", "1"));
			messageProducer.setDeliveryMode(deliveryMode);
			TextMessage outMessage = session.createTextMessage();
			outMessage.setText(message);
			outMessage.setJMSDestination(_destinationQueue);
			messageProducer.send(outMessage);
		} finally {
			if (messageProducer != null)
				messageProducer.close();
		}
	}
}