/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.exception;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.coordinator.CoordinatorIFace;
import it.eng.spago.dispatching.coordinator.DispatcherManager;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.dispatching.service.ServiceIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.init.InitializerIFace;
import it.eng.spago.tracing.TracerSingleton;

import java.io.Serializable;

/**
 * <code>EMFExceptionHandler</code> is a configurable manager for exceptions 
 * thrown by services.<br/>
 * Through the definition of ERRORs and TRAPs in the configuration is possible
 * to define a service that manage an exception category; than is possible to
 * map an Exception to a particular exception category.
 * <br/>
 * <code>EMFExceptionHandler</code> check if a ERROR is available for a given
 * Exception otherwise try to use the ERROR with the <code>UNDEFINED</code>
 * class. If the <code>UNDEFINED</code> ERROR is not defined exception is logged
 * and added to the EMFErrorHandler.
 * 
 * @author buso
 *
 */
public class EMFExceptionHandler implements Serializable {
    private static final long serialVersionUID = 1L;
	
	// Have you to put all contants in Constants class?
    /**
     * Path to identify a ERROR element in the configuration.
     */
	public static final String ERROR_PATH = "ERRORS.ERROR";
    /**
     * Path to identify a TRAP element in the configuration.
     */
	public static final String TRAP_PATH = "TRAPS.TRAP";
	/**
	 * Path to identify a TRAP element into a service configuration.
	 * <br/>
	 * NOTE: this imply <code>ACTION</code> and <code>PAGE</code> must have equal
	 * <code>TRAP</code> configurations.
	 */
	public static final String CONFIG_SERVICE_TRAP_PATH = "TRAPS.TRAP"; 
	/**
	 * Used in ERROR element to configure the TRAP
	 * that will handle all exceptions not directly mapped as ERRORs.
	 */
	public static final String ERROR_UNDEFINED_CLASS = "UNDEFINED";
	
	
	/**
	 * Retrive the TRAP <code>SourceBean</code> configured or null if no
	 * configuration for the given Exception is defined.
	 * 
	 * @param e. The Exeption to search for in configuration.
	 * @return the TRAP <code>SourceBean</code> configured for <code>e</code>.
	 */
	private SourceBean getTrapSourceBean(Exception e, ServiceIFace service) {
		ConfigSingleton configSingleton = ConfigSingleton.getInstance();
		SourceBean error = (SourceBean)configSingleton.getFilteredSourceBeanAttribute(
			ERROR_PATH, "CLASS", e.getClass().getName());
		if (error == null) {
			// try with the ERROR_UNDEFINED_CLASS one
			error = (SourceBean)configSingleton.getFilteredSourceBeanAttribute(
					ERROR_PATH, "CLASS", ERROR_UNDEFINED_CLASS);
			if (error == null) { return null; }
		} // if (error == null) {
		String errorTrap = (String)error.getAttribute("CATEGORY");
		if (errorTrap == null) {
			// error in configuration throw some exception?
			return null;
		} // if (errorTrap == null)
		SourceBean trap = null;
		// search for the trap in the ServiceIFace configuration
		trap = getTrapFromService(service, errorTrap);
		if (trap == null) {
			// otherwise search for the trap into the configuration
			trap = (SourceBean)configSingleton.getFilteredSourceBeanAttribute(
				TRAP_PATH, "CATEGORY", errorTrap);
		}//if (trap == null)
		
		return trap;
	} // private SourceBean getTrapSourceBean(Exception e)
	
	private SourceBean getTrapFromService(ServiceIFace service, String trapCategory) {
		if (service == null) { return null; }
		SourceBean trap = null;
		if (service instanceof InitializerIFace) {
			SourceBean serviceConfig = ((InitializerIFace)service).getConfig();
			if (serviceConfig != null) {
				trap = (SourceBean)serviceConfig.getFilteredSourceBeanAttribute(
					CONFIG_SERVICE_TRAP_PATH, "CATEGORY", trapCategory);
		        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	            	"EMFExceptionHandler::getTrapFromService: specific trap for: service ["+
	            	service.getClass().getName()+"], " +
	            	"trap category ["+trapCategory+"], "
	        		+"trap ["+((trap != null)?trap.toString():null)+"]");
			}//if (serviceConfig != null)
		}//if (service instanceof InitializerIFace)
		return trap;
	}//private SourceBean getTrapFromService(ServiceIFace service, String trapCategory)

	/**
	 * Handle the exception using the ERROR --> TRAP configuration. If no ERROR
	 * is found for a given Exception it search for the <code>UNDEFINED</code>
	 * ERROR, otherwise only a log is performed.
	 * It need the <code>EMFErrorHandler</code> correctly setted in
	 * <code>ResponseContainer</code>. 
	 * 
	 * @param e. The Exception to search for in the configuration.
	 * @param requestContext. Used to execute the service associated with
	 * the configured TRAP.
	 */
	public void handleException(Exception e, ServiceIFace service, RequestContextIFace requestContext) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
        	"EMFExceptionHandler::handleException: invoked; exception: "+e.getClass().getName());
		
        // put the exception in emfErrorHandler
        requestContext.getResponseContainer().getErrorHandler().addError(
        	new EMFInternalError(EMFErrorSeverity.ERROR, e));
        
		// obtain the right TRAP
		SourceBean trap = getTrapSourceBean(e, service);
		// if you can not found neither the specific TRAP neither the
		// ERROR_UNDEFINED_CLASS one, use EMFErrorHandler behaviour
		if (trap == null) {
	        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	            	"EMFExceptionHandler::handleException trap not found for ("
	            		+e.getClass().getName()+")");
			doEMFErrorHandlerBehaviour(e, requestContext, EMFErrorSeverity.ERROR);
			return;
		} // if (trap == null)
		// change the request to dispatch to the founded TRAP
		RequestContainer requestContainer = requestContext.getRequestContainer();
		SourceBean serviceRequest = requestContainer.getServiceRequest();
		String trapType = (String)trap.getAttribute("TRAP_TYPE");
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
        	"EMFExceptionHandler::handleException ("
        		+e.getClass().getName()+"): TRAP_TYPE: "+trapType);
		
		try {
			
			// remove fault service ACTION_NAME and PAGE

			// TODO check if you need to save fault service name to permit
			// to trace it in some way
			
			serviceRequest.delAttribute(Constants.ACTION_NAME);
			serviceRequest.delAttribute(Constants.PAGE);
			if (trapType.equals("ACTION")) {
				serviceRequest.setAttribute(Constants.ACTION_NAME,
					(String)trap.getAttribute("TRAP_NAME"));
			} // if (trapType.equals("ACTION"))
			if (trapType.equals("PAGE")) {
				serviceRequest.setAttribute(Constants.PAGE,
					(String)trap.getAttribute("TRAP_NAME"));
			} // if (trapType.equals("PAGE"))
			
			// request the coordinator to execute the TRAP
	        CoordinatorIFace coordinator = DispatcherManager.getCoordinator(requestContext);
	        Exception serviceException = null;
	    	ResponseContainer responseContainer = requestContext.getResponseContainer();
	        if (coordinator == null) {
	            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
	                    "EMFExceptionHandler::handleException: coordinator nullo !");
	            serviceException = new Exception("Coordinatore non trovato");
	            responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR,
	                    "Coordinatore non trovato !", serviceException));
	        } // if (coordinator == null)
	        else {
	            ((RequestContextIFace) coordinator).setRequestContext(requestContext);
	            responseContainer.setBusinessType(coordinator.getBusinessType());
	            responseContainer.setBusinessName(coordinator.getBusinessName());
	            responseContainer.setPublisherName(coordinator.getPublisherName());
	
	                coordinator.service(serviceRequest,
	                	responseContainer.getServiceResponse());

	            responseContainer.setAttribute(Constants.ERROR_MANAGED, "TRUE");
	
	        } // if (coordinator == null) else
		} catch (Exception ex) {
			// if exception trace and use ErrorHandler
			doEMFErrorHandlerBehaviour(ex, requestContext, EMFErrorSeverity.BLOCKING);
		} // catch (Exception ex)
	}
	
	/**
	 * Log the exception and add it to the EMFErrorHandler.
	 * 
	 * @param e
	 * @param requestContext
	 */
	private void doEMFErrorHandlerBehaviour(Exception e,
		RequestContextIFace requestContext, String severity) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "EMFExceptionHandler::handleException:", e);
        // this code is specific for the AdapterHTTP. Check on other adapters if
        // all is setted/initialized in the same way
        EMFErrorHandler emfErrorHandler = requestContext.getResponseContainer()
        	.getErrorHandler(); 
        emfErrorHandler.addError(new EMFInternalError(severity, e));
	} // private void doEMFErrorHandlerBehaviour(Exception e, RequestContextIFace requestContext)
}
