/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.io.os400;

import it.eng.spago.base.Constants;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import com.ibm.as400.access.AS400;
import com.ibm.as400.access.AS400ConnectionPool;
import com.ibm.as400.access.ConnectionPoolException;

/**
 * 
 * Classe per la gestione del pool di connessioni verso OS/400
 * @author grassi11
 *
 */
public class OS400AdapterResourcePool implements Serializable {
	private static final long serialVersionUID = -4258921569231147732L;
	private static OS400AdapterResourcePool _instance = null;
	protected static int maxConnection = 10; // Numero massimo di collegamenti per questo lotto
	protected static int minConnection = 1; // Numero minimo di collegamenti per questo lotto
	//protected static long maxLifeTime = 1000 * 60 * 60; // Durata massima in milliSecondi per collegamento
	protected static long maxLifeTime = -1; // Durata massima in milliSecondi per collegamento
	/*
	protected static String systemName = "10.10.2.181";  
	protected static String userId = "SYNON";
	protected static String password = "PINO";
*/	
	private AS400ConnectionPool pool;
	
	private class ServerInfo {
		public String systemName ;
		public String userId ;
		public String password ;
	}
	private static ServerInfo _serverInfo = null ;

	private void lookupServerInfo() throws Exception {
		_serverInfo = null ;
		try {
			InitialContext iniCtx = new InitialContext();
			_serverInfo = new ServerInfo() ;
			_serverInfo.systemName = (String)iniCtx.lookup("OS400_IP");
			_serverInfo.userId = (String)iniCtx.lookup("OS400_USER");
			_serverInfo.password = (String)iniCtx.lookup("OS400_PASSWORD");
		} catch (NamingException ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "OS400AdapterResourcePool::lookupServerInfo:Errore durante il recupero da spazio nomi WebSphere dei riferimenti al server OS400", (Exception) ex);
			throw ex ;
		}
		
	}
	
	private OS400AdapterResourcePool() throws Exception {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "OS400AdapterResourcePool: creating pool");
		pool = new AS400ConnectionPool();
		pool.setMaxConnections(maxConnection);
		pool.setMaxLifetime(maxLifeTime);
		// Precollegare 1 collegamenti al servizio AS400.COMMAND.
		try {
			lookupServerInfo() ;
			pool.fill(_serverInfo.systemName, _serverInfo.userId, _serverInfo.password, AS400.COMMAND, minConnection);
// System.out.println("Preconnected 1 connection to the AS400.COMMAND service");
		} catch (Exception ex) {		
			TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "OS400AdapterResourcePool::getConnection:il pool ha ritornato errore", ex);
            throw  ex ;
		}
	}

	/**
	 * Ritorna una connessione dal pool
	 * 
	 * @return
	 */
	public synchronized AS400 getConnection() throws ConnectionPoolException {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                            "OS400AdapterResourcePool::getConnection");
		AS400 connection = null ;
		try {
			connection = pool.getConnection(_serverInfo.systemName, _serverInfo.userId, _serverInfo.password, AS400.COMMAND);
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
						  "OS400AdapterResourcePool::getConnection - Number of active connections: "
					    + pool.getActiveConnectionCount(_serverInfo.systemName, _serverInfo.userId));
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					  "OS400AdapterResourcePool::getConnection - Number of available connections for reuse: "
						+ pool.getAvailableConnectionCount(_serverInfo.systemName, _serverInfo.userId));
		} catch (ConnectionPoolException ex) {		
			TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "OS400AdapterResourcePool::getConnection:il pool ha ritornato errore", ex);
            throw ex ;
		}
   		return connection;
		
	}

	/**
	 * Ritorna l'istanza della classe (Singleton)
	 * 
	 * @return
	 */
	public static OS400AdapterResourcePool getInstance() {
		if (_instance == null) {
			synchronized (OS400AdapterResourcePool.class) {
				if (_instance == null) {
					try {
						_instance = new OS400AdapterResourcePool();
					} // try
					catch (Exception ex) {
						TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "OS400AdpaterResourcePool::getInstance:Error",ex);
						//ex.printStackTrace();
					} // catch(Exception ex) try
				} // if (_instance == null)
			}
		} // if (_instance == null)
		return _instance;
	}
	
	/*
	 * Rilascia una connessione al pool
	 */
	public void closeConnection(AS400 conn) {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
        	"OS400AdapterResourcePool::closeConnection");
		
		pool.returnConnectionToPool(conn) ;
		
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				  "OS400AdapterResourcePool::getConnection - Number of active connections: "
			    + pool.getActiveConnectionCount(_serverInfo.systemName, _serverInfo.userId));
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
			  "OS400AdapterResourcePool::getConnection - Number of available connections for reuse: "
				+ pool.getAvailableConnectionCount(_serverInfo.systemName, _serverInfo.userId));
	}

	/**
	 * Chiude l'eventuale pool aperto e rilascia l'istanza di questa classe
	 */
	public static void release() {
		if (_instance != null) {
			synchronized (OS400AdapterResourcePool.class) {
				try {
					_instance.finalize();
				} // try
				catch (Throwable ex) {
					TracerSingleton.log(
			                Constants.NOME_MODULO,
			                TracerSingleton.CRITICAL,
			                "OS400AdapterResourcePool::release: " + ex.getMessage());
				} // catch(Exception ex) try
				_instance = null;
			}
		} // if (_instance != null)
	} // public static void release()

	/**
	 * Se aperto, chiude prima il pool
	 */
	protected void finalize() throws Throwable {
		if (pool != null) {
			pool.close();
		}
		super.finalize();
	}
}