/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.paginator.smart.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.paginator.smart.AbstractRowProvider;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.QueryExecutor;
import it.eng.spago.util.StatementProvider;
import it.eng.spago.util.StringUtils;
import it.eng.spago.validation.EMFValidationError;

import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

/**
 * La classe <code>DBRowProvider</code> estende
 * <code>AbstractRowProvider</code> ed implementa i metodi di accesso alle
 * sorgenti di dati riconducibili al pacchetto dbaccess del framework.
 * <p>
 * @version 2.0, 14/03/2007
 * @see AbstractRowProvider
 */
 

public class DBRowProvider extends AbstractRowProvider {
    private static final long serialVersionUID = 1L;

    private static final String SELECT_WARNING_MAX_ROWS = "10126";
    
    private DataConnection _dataConnection = null;
    private SQLCommand _sqlCommand = null;
    private DataResult _dataResult = null;


    /**
     * Costruisce un <code>DBRowProvider</code>. Questo costruttore &egrave;
     * vuoto ed ogni azione di inizializzazione &egrave; demandata al metodo
     * <code>init(SourceBean config)</code>.
     * <p>
     * @see DBRowProvider#init(SourceBean)
     */
    public DBRowProvider() {
        super();
        _dataConnection = null;
        _sqlCommand = null;
        _dataResult = null;
    } // public DBRowProvider()

    /**
     * Questo metodo apre la connessione alla sorgente dati e recupera tutte le
     * righe relative alla query configurata.
     * <p>
     * @see DBRowProvider#init(SourceBean)
     */
    public void open() {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DBRowProvider::open: invocato");
        super.open();
        SourceBean config = getConfig();
        SourceBean query = (SourceBean) config.getAttribute("LIST_QUERY");
        String pool = (String) config.getAttribute("POOL");
        
        try {
            DataConnectionManager dataConnectionManager =  DataConnectionManager.getInstance();
            _dataConnection = dataConnectionManager.getConnection(pool, _requestContainer);
            StatementProvider statementProvider = new StatementProvider(this, _dataConnection, query) ;
            String statement = statementProvider.getStatement() ;
            List inputParameters = statementProvider.getParameterList() ;
            _sqlCommand = _dataConnection.createSelectCommand(statement);
            if (inputParameters != null)
            	_dataResult = _sqlCommand.execute(inputParameters);
            else 
            	_dataResult = _sqlCommand.execute();
            if(_sqlCommand.getMaxRowsReached()) {
    			EMFErrorHandler errorHandler = this.getErrorHandler();
    			errorHandler.addError(new EMFValidationError(EMFErrorSeverity.ERROR,"NONAME",SELECT_WARNING_MAX_ROWS));
            }
            ScrollableDataResult scrollableDataResult = (ScrollableDataResult) _dataResult.getDataObject();
            setRows(scrollableDataResult.getRowsNumber());
            setCurrentRow(0);
        } // try
        catch (Exception ex) {
        	getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, (ex instanceof EMFInternalError) ? ((EMFInternalError) ex).getNativeException() : ex));
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DBRowProvider::executeQuery:",
                ex);
            Utils.releaseResources(_dataConnection, _sqlCommand, _dataResult);
            _dataConnection = null;
            _sqlCommand = null;
            _dataResult = null;
        } // catch (Exception ex)
    } // public void open()

    /**
     * Questo metodo serve per posizionare il <em>cursore</em> di lettura
     * alla riga individuata da <em>row</em>.
     * <p>
     * @param row <code>int</code> il numero della riga su cui posizionarsi.
     */
    public void absolute(int row) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DBRowProvider::absolute: invocato");
        if (_dataResult == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DBRowProvider::absolute: row provider non aperto correttamente !");
            return;
        } // if (_dataResult == null)
        if (row < 1)
            row = 1;
        else if (row > rows())
            row = rows();
        setCurrentRow(row - 1);
        ScrollableDataResult scrollableDataResult =
            (ScrollableDataResult) _dataResult.getDataObject();
        try {
            scrollableDataResult.moveTo(row);
        } // try
        catch (EMFInternalError emfie) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DBRowProvider::absolute:",
                (Exception) emfie);
        } // catch (EMFInternalError emfie)
    } // public void absolute(int row)

    /**
     * Ritorna un <code>Object</code> che rappresenta la riga individuata da
     * <em>row</em>.
     * <p>
     * @return <code>Object</code> la riga individuata da <em>row</em>.
     * @param row <code>int</code> il numero della riga da ritornare.
     */
    public Object getRow(int row) {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DBRowProvider::getRow: invocato");
        if (_dataResult == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DBRowProvider::getRow: row provider non aperto correttamente !");
            return null;
        } // if (_dataResult == null)
        if (row < 1)
            row = 1;
        else if (row > rows())
            row = rows();
        setCurrentRow(row);
        ScrollableDataResult scrollableDataResult =
            (ScrollableDataResult) _dataResult.getDataObject();
        SourceBean rowBean = null;
        try {
            rowBean = scrollableDataResult.getDataRow(row).getSourceBean();
        } // try
        catch (EMFInternalError emfie) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DBRowProvider::getRow:",
                (Exception) emfie);
        } // catch (EMFInternalError emfie)
        return rowBean;
    } // public Object getRow(int row)

    /**
     * Ritorna un <code>Object</code> che rappresenta la riga successiva
     * all'ultima recuperata o individuata con il comando <code>absolute(int)
     * </code>.
     * <p>
     * @return <code>Object</code> la riga individuata da <em>row</em>.
     * @param row <code>int</code> il numero della riga da ritornare.
     */
    public Object getNextRow() {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DBRowProvider::getNextRow: invocato");
        if (_dataResult == null) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "DBRowProvider::getNextRow: row provider non aperto correttamente !");
            return null;
        } // if (_dataResult == null)
        ScrollableDataResult scrollableDataResult =
            (ScrollableDataResult) _dataResult.getDataObject();
        SourceBean rowBean = null;
        try {
            if (scrollableDataResult.hasRows()) {
                rowBean = scrollableDataResult.getDataRow().getSourceBean();
                setCurrentRow(getCurrentRow() + 1);
            } // if (scrollableDataResult.hasRows())
        } // try
        catch (EMFInternalError emfie) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.CRITICAL,
                "DBRowProvider::getNextRow:",
                (Exception) emfie);
        } // catch (EMFInternalError emfie)
        return rowBean;
    } // public Object getNextRow()

    /**
     * Questo metodo chiude la connessione alla sorgente dati.
     * <p>
     * @see DBRowProvider#init(SourceBean)
     */
    public void close() {
        TracerSingleton.log(
            Constants.NOME_MODULO,
            TracerSingleton.DEBUG,
            "DBRowProvider::close: invocato");
        Utils.releaseResources(_dataConnection, _sqlCommand, _dataResult);
        _dataConnection = null;
        _sqlCommand = null;
        _dataResult = null;
   /* AG: 07/05/2007
        setRows(0);
   */
        setCurrentRow(0);
    } // public void close()
    
}
