/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.services;

import java.util.Iterator;
import java.util.List;

import org.apache.commons.validator.GenericValidator;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;

/**
 * Manage a service catalog. Common services can be putted (configured) in this
 * catalog and their lifecycle will be managed by <code>ServiceCatalog</code>.
 * 
 * A service has a scope that determine its lifecycle management.  
 * 
 */
public class ServiceCatalog implements ServiceCatalogIFace {
	/**
	 * Configuration path used to retrive services configuration.
	 */
	private static final String SERVICE_PATH = "SERVICES.SERVICE";

	/**
	 * Prefix key used to store an instatiated service into the context. 
	 */
	private static final String SERVICE_KEY_PREFIX = "service_";
	
	private static Boolean servicesInitialized = Boolean.FALSE;
	
	/**
	 * Singleton instance.
	 */
	private static ServiceCatalog _instance = null;

    /**
     *  Constructor for the ServiceCatalog object
     */
//    protected ServiceCatalog(ServletContext ctx) throws ServletException, ServiceException {
    private ServiceCatalog() {
    }// private ServiceCatalog() {
        
    /**
     * Get the singleton instance.
     * 
     * @return
     */
    public static ServiceCatalog getInstance() {
    	if (_instance == null) {
    		_instance = new ServiceCatalog();
    	}// if (_instance == null)
    	return _instance;
    }// public static ServiceCatalog getInstance()

    public void initServices() throws EMFInternalError, ServiceNotFound {
    	synchronized (servicesInitialized) {
    		if (!servicesInitialized.booleanValue()) {
    			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
    				"ServiceCatalog::initServices: initializing...");
				List servicesConfig = ConfigSingleton.getInstance()
					.getAttributeAsList(SERVICE_PATH);
				
				if (servicesConfig != null) {
					Iterator servicesCfgIter = servicesConfig.iterator();
					while (servicesCfgIter.hasNext()) {
						SourceBean srvCfg = (SourceBean)servicesCfgIter.next();
						String srvScope = (String)srvCfg.getAttribute("SCOPE");
						if (!GenericValidator.isBlankOrNull(srvScope)
							&& srvScope.equals("APPLICATION")) {
							
			    			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
			    				"ServiceCatalog::initServices: service to initialize: "+srvCfg.toString());
							String serviceName = (String)srvCfg.getAttribute("NAME");
					    	ServiceInterface service = instantiateService(srvCfg);
					    	ApplicationContainer.getInstance().setAttribute(
					    		serviceName, service);
						}
					}//while (servicesCfgIter.hasNext())
				}//if (servicesConfig != null)
				servicesInitialized = Boolean.TRUE;
    		}//if (!servicesInitialized.booleanValue())
		}//synchronized (servicesInitialized)
    }//public void initServices()
    
    /**
     * 
     * @see ServiceCatalogIFace#getService(RequestContextIFace, String)
     */
//    public ServiceInterface getService(String name) throws ServiceException {
    public ServiceInterface getService(RequestContextIFace requestContext,
    	String name) throws Exception {
    	
    	ServiceInterface toReturn = null;
    	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, 
        		"ServiceCatalog::getService: looking for service with name: "+name);
    	
    	SourceBean serviceBean = getServiceConfigBean(name);

    	if (serviceBean == null) {
    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, 
    			"ServiceCatalog::getService: Service with name: "+name+" not found.");
    		throw new ServiceNotFound(name);
    	}// if (serviceBean == null)
    	
    	String serviceScope = (String)serviceBean.getAttribute("SCOPE");
    	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, 
    		"ServiceCatalog::getService: Service scope is: "+serviceScope);
    	
    	if (serviceScope.equals("REQUEST")) {
    		return instantiateService(serviceBean);
    	}// if (serviceScope.equals("REQUEST"))
    	String serviceContextKey = getServiceCtxKey(name);
    	
    	ServiceInterface serviceFromCtx = (ServiceInterface)ContextScooping
    		.getScopedParameter(requestContext.getRequestContainer(),
    				requestContext.getResponseContainer(),
    				serviceContextKey, serviceScope);
    	
    	if (serviceFromCtx != null) {
    		toReturn = serviceFromCtx;
    	} else {
	    	toReturn = instantiateService(serviceBean);
	    	// this code should be in ContextScooping?
	    	if (serviceScope.equals("SESSION")) {
	    		requestContext.getRequestContainer().getSessionContainer().getPermanentContainer().setAttribute(serviceContextKey,
	    				toReturn);
	    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	        			"ServiceCatalog::getService: added service ("+name+") in SESSION scope");
	    	} else if (serviceScope.equals("APPLICATION")) {
	    		ApplicationContainer.getInstance().setAttribute(serviceContextKey,
	    				toReturn);
	    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	        			"ServiceCatalog::getService: added service ("+name+") in APPLICATION scope");
	    	} else {
	    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	    			"ServiceCatalog::getService: Unknown service scope: "+serviceScope);
	    	}// if (serviceScope.equals("SESSION"))
    	}// if (serviceFromCtx != null)
    	return toReturn;
    }// public ServiceInterface getService(RequestContextIFace requestContext, String name) throws Exception

    private SourceBean getServiceConfigBean(String name) {
    	ConfigSingleton config = ConfigSingleton.getInstance();
    	return (SourceBean)config.getFilteredSourceBeanAttribute(
    		SERVICE_PATH, "name", name);
    }// private SourceBean getServiceConfigBean(String name)
    
    private ServiceInterface instantiateService(SourceBean serviceConfig)
    	throws ServiceNotFound, EMFInternalError {
    	
    	synchronized (ServiceCatalog.class) {
	    	ServiceInterface instantiatedService = null;
	    	String serviceName = (String)serviceConfig.getAttribute("NAME");
	    	String classString = (String)serviceConfig.getAttribute("CLASS");
	    	
	    	try {
				Class clazz = Class.forName(classString);
				instantiatedService = (ServiceInterface)clazz.newInstance();
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
					"ServiceCatalog::instantiateService: instantiated service: "
					+classString);
				((ServiceInterface)instantiatedService).init(
					(SourceBean)serviceConfig.getAttribute("CONFIG"));
				((ServiceInterface)instantiatedService).setScope(
						(String)serviceConfig.getAttribute("SCOPE"));
			} catch (ClassNotFoundException e) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
					"ServiceCatalog::instantiateService: Service class not found", e);
				ServiceNotFound snf = new ServiceNotFound(serviceName);
				snf.initCause(e);
				throw snf;
			} catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
						"ServiceCatalog::instantiateService:", ex);
				throw new EMFInternalError(EMFErrorSeverity.ERROR, ex);
			}//try
	    	
	    	return instantiatedService;
		}

    }//private ServiceInterface instantiateService(SourceBean serviceConfig)
//	throws ServiceNotFound, EMFInternalError

    /**
     *  Gets the serviceList attribute of the ServiceCatalog object
     *
     *@return    The serviceList value
     */
//    public Enumeration getServiceList() {
//        return services.getServiceList();
//    }

    /**
     * 
     * @see ServiceCatalogIFace#isServiceAvailable(String)
     */
    public boolean isServiceAvailable(String name) {
        return getServiceConfigBean(name) != null;
    }//public boolean isServiceAvailable(String name)
    
    private String getServiceCtxKey(String name) {
    	return new StringBuffer(SERVICE_KEY_PREFIX)
			.append(name).toString();
    }
    
}// public class ServiceCatalog implements ServiceCatalogIFace
