/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.services;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.util.ContextScooping;

import java.io.Serializable;
import java.net.URL;
import java.net.UnknownHostException;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

/**
 * Servizio generalizzato per invocare servizi attraverso AdapterAxis
 *
 * 	<SERVICE name="SrvProtocollo" class="it.eng.spago.service.ServiceProxyAxis" SCOPE="REQUEST">
 *		<REQUEST>
 *			<PARAMETER name="ACTION_NAME" type="ABSOLUTE" value="ActGetNextProtocollo"/>
 *			<PARAMETER name="AMBIENTE" type="ABSOLUTE" value="Sied"/>
 *		</REQUEST>
 *	</SERVICE>
 *
 * @author grassi
 */
public class ServiceProxyAxis extends AbstractService implements Serializable {
	private static final long serialVersionUID = -7852616393152406862L;
	public final static String SERVICE_NAME_SPACE = "http://axis.soap.spago.eng.it" ;
	public final static String SERVICE_NAME = "service" ;
	
	private SourceBean _config ;
	private SourceBean _serviceRequest, _serviceErrors ;
	private String _endPointURL ;
	
	public void init(SourceBean config) throws Exception {
		_config = config ;
		_serviceRequest = null ;
		_serviceErrors = null ;
		_endPointURL=getServerIP((String)_config.getAttribute("serverURL") )+ _config.getAttribute("endPoint") ;
		
		super.init(config);
	}
	
	/**
	 * Inizializza la richiesta per il servizio da invocare
	 * i parametri sono letti attraverso la configurazione (services.xml)
	 * es: <PARAMETER name="LISTA_VUOTA" type="RELATIVE", scope="SERVICE_RESPONSE" /> 
	 * @throws SourceBeanException 
	 */
	private void loadServiceRequest(RequestContextIFace requestContext) throws SourceBeanException {
		_serviceRequest = new SourceBean("SERVICE_REQUEST") ;
		for (Object par : _config.getAttributeAsList("REQUEST.PARAMETER")) {
			Object value = ContextScooping.getScopedParameter(requestContext, (SourceBean)par)  ;
			if (value != null) {
				String key = (String)((SourceBean)par).getAttribute("NAME") ;
				_serviceRequest.setAttribute(key, value) ;
			}
		}
	}
	
	/**
	 * Permette di aggiungere parametri alla SERVICE_REQUEST per l'invocazione del servizio
	 * @param key
	 * @param value
	public void addServiceRequestParameter(String key, Object value) {
		_serviceRequest.setAttribute(key, value) ;
	}
	 */

	/**
	 * Ritorna l'indirizzo del server in esecuzione
	 * @return
	 * @throws UnknownHostException
	 */
	private String getServerIP(String serverIP) throws UnknownHostException {
		if (serverIP == null || serverIP.trim().length() == 0)
			return "http://" + java.net.InetAddress.getLocalHost().getHostAddress();
		else
			return serverIP ;
	}
	
	/**
	 * Invoca il servizio richiesto
	 * @param requestContext
	 * @return SourceBean con la risposta del servizio eseguito
	 * @throws Exception
	 */
	public Object invoke(RequestContextIFace requestContext) throws Exception {
		loadServiceRequest(requestContext) ;
		_serviceErrors = null ;
		Call call = (Call) new Service().createCall();
		call.setTargetEndpointAddress(new URL(_endPointURL));
		call.setOperationName(new javax.xml.namespace.QName(SERVICE_NAME_SPACE, SERVICE_NAME));
		String wsResponse = (String) call.invoke(new java.lang.Object[] { _serviceRequest.toString(false,false) });
		
		return handleResponse(requestContext,  SourceBean.fromXMLString(wsResponse)) ;
	}
	
	/**
	 * Estrae la risposta del servizio eseguito e carica gli eventuali errori di esecuzione
	 * @param requestContext
	 * @param serviceResponse  ritorno dal servizio eseguito
	 *      es: <RESPONSE><SERVICE_RESPONSE><ROW nextprotocollo="#1001623#"/></SERVICE_RESPONSE><ERRORS/></RESPONSE>
	 */
	private Object handleResponse(RequestContextIFace requestContext, SourceBean wsResponse) throws Exception {
		_serviceErrors = new SourceBean((SourceBean)wsResponse.getAttribute("ERRORS")) ;
		return wsResponse.getAttribute("SERVICE_RESPONSE.ROW") ;
	}

	/**
	 * @return Restituisce il valore _serviceErrors.
	 */
	public SourceBean getServiceErrors() {
		return _serviceErrors;
	}
}