/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.services.fop;

import it.eng.spago.base.ApplicationContainer;
import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.presentation.StylesheetFile;
import it.eng.spago.services.AbstractService;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.commons.validator.GenericValidator;
import org.apache.fop.apps.Driver;
import org.apache.fop.messaging.MessageHandler;

/**
 * Service that convert an xml stream into other formats using
 * <a href="http://xmlgraphics.apache.org/fop/index.html">FOP</a>
 * (an xsl-fo processor provided by apache).
 * <br/>
 * 
 * The service accept in the CONFIG envelop the attribute <code>log_level</code>
 * used to set the logger passed to FOP.
 * Valid values are: DEBUG, DISABLED, ERROR, FATAL, INFO, WARN.
 * 
 * NOTE: the version used is the 0.20.5 newer versions (&gt;0.91) have
 * different API.
 * 
 * @author buso
 *
 */
public class FOPService extends AbstractService {
	private static final long serialVersionUID = -2256312172145788824L;
	public static final String PDF_CONTENT_TYPE = "application/pdf";
	public static final String AWT_CONTENT_TYPE = "application/X-fop-awt-preview";
	public static final String MIF_CONTENT_TYPE = "application/mif";
	public static final String XML_CONTENT_TYPE = "text/xml";
	public static final String PRINT_CONTENT_TYPE = "application/X-fop-print";
	public static final String PCL_CONTENT_TYPE = "application/x-pcl";
	public static final String PS_CONTENT_TYPE = "application/postscript";
	public static final String TEXT_CONTENT_TYPE = "text/plain";
	public static final String SVG_CONTENT_TYPE = "image/svg+xml";
	
	private static final Map renderTypeToContentType = new HashMap();
	
	private String logLevel;
	
	static {
		renderTypeToContentType.put(PDF_CONTENT_TYPE, new Integer(Driver.RENDER_PDF));
		renderTypeToContentType.put(AWT_CONTENT_TYPE,new Integer(Driver.RENDER_AWT));
		renderTypeToContentType.put(MIF_CONTENT_TYPE, new Integer(Driver.RENDER_MIF));
		renderTypeToContentType.put(XML_CONTENT_TYPE, new Integer(Driver.RENDER_XML));
		renderTypeToContentType.put(PRINT_CONTENT_TYPE,new Integer(Driver.RENDER_PRINT));
		renderTypeToContentType.put(PCL_CONTENT_TYPE, new Integer(Driver.RENDER_PCL));
		renderTypeToContentType.put(PS_CONTENT_TYPE, new Integer(Driver.RENDER_PS));
		renderTypeToContentType.put(TEXT_CONTENT_TYPE,new Integer(Driver.RENDER_TXT));
		renderTypeToContentType.put(SVG_CONTENT_TYPE, new Integer(Driver.RENDER_SVG));
	}

	public void init(SourceBean config) {
		if (config != null) {
			logLevel = (String)config.getAttribute("log_level");
		}
		if (GenericValidator.isBlankOrNull(logLevel)) {
			logLevel = "INFO";
		}

	}
	
	/**
	 * Execute an xsl transformation to obtain an xsl-fo document from the
	 * <code>xml stream</code> passed as parameter. The obtained xsl-fo is
	 * processed by FOP to be rendered in the <code>contentType</code> format
	 * passed as parameter.
	 * 
	 * @param xml. The <code>xml stream</code> containing the content to be rendered
	 * 		with FOP.
	 * @param xslName. Name of the <code>xsl</code> configured in publishers.xml
	 * 		used to transform the <code>xml stream</code> passed to xsl-fo. 
	 * @param contentType. Has to be one of the content type supported
	 * by <a href="http://xmlgraphics.apache.org/fop/0.20.5/index.html" a>FOP</a>
	 * redefined in this class as constants. 
	 * @param out. Stream where FOP will render the document.
	 * @throws TransformerException if errors occur during xsl transformation
	 * or FO processing.
	 */
	public void process(InputStream xml, String xslName,
		String contentType, OutputStream out, RequestContextIFace requestCtx)
		throws TransformerException {

        if (xml == null) {
        	TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
        		"FOPService::process: passed xml is null.");
        	return;
        }
        
		ApplicationContainer application = ApplicationContainer.getInstance();
		Hashtable xslTemplates = (Hashtable) application
			.getAttribute("XSL_TEMPLATES");
		if (xslTemplates == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"FOPService::process: xslTemplates nullo");
			return ;
		} // if (xslTemplates == null)
		// Retrive from cache the file identified by the label xslName
		StylesheetFile stylesheet = (StylesheetFile) xslTemplates
			.get(xslName);
		if (stylesheet == null) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"FOPService::process: template di stylesheetName ["
							+ xslName + "] non trovato");
			return;
		} // if (stylesheet == null)
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
				"FOPService::process: template di stylesheetName ["
				+ xslName + "]");
		// Retrive the template rappresenting the processed xsl transformation
		Templates template = stylesheet.getTemplate();
		try {
			Transformer transformer = template.newTransformer();
			Source src = new StreamSource(xml);
			// set FOP driver
			Driver driver = new Driver();
			MessageHandler.setScreenLogger(new ConsoleLogger(getLogLevel()));
			MessageHandler.setQuiet(true);
			driver.setLogger(new ConsoleLogger(getLogLevel()));
			driver.setRenderer(getFOPRenderType(contentType));
			driver.setOutputStream(out);
			
			Result res = new SAXResult(driver.getContentHandler());
			transformer.transform(src, res);
		} //try
		catch (TransformerConfigurationException e) {
//			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
//					"FOPService::process: ", e);
			throw e;
		} catch (TransformerException e) {
//			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
//					"FOPService::process: ", e);
			throw e;
		}
	}
	
	private int getLogLevel() {
		if (logLevel.equals("DEBUG")) { return ConsoleLogger.LEVEL_DEBUG; }
		if (logLevel.equals("DISABLED")) { return ConsoleLogger.LEVEL_DISABLED; }
		if (logLevel.equals("ERROR")) { return ConsoleLogger.LEVEL_ERROR; }
		if (logLevel.equals("FATAL")) { return ConsoleLogger.LEVEL_FATAL; }
		if (logLevel.equals("INFO")) { return ConsoleLogger.LEVEL_INFO; }
		if (logLevel.equals("WARN")) { return ConsoleLogger.LEVEL_WARN; }
		return ConsoleLogger.LEVEL_INFO;
	}
	
	/**
	 * Utility method that return the content type to use with a given
	 * render type defined in {@link Driver}.
	 * 
	 * @param renderType
	 * @return
	 */
	private int getFOPRenderType(String contentType) {
		Integer renderType = (Integer)renderTypeToContentType.get(contentType);
		
		return renderType != null? renderType.intValue() : -1;
	}
}
