/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.services.mail;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.services.AbstractService;
import it.eng.spago.tracing.TracerSingleton;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.Address;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message.RecipientType;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.apache.commons.validator.GenericValidator;

/**
 * This service send an email through a configured smtp server.
 * The execution is done using javamail package.
 * 
 * <br/>
 * Configuration example:
 * <pre>
 *	&lt;SERVICE NAME="sendMail"
 *		CLASS="it.eng.spago.services.mail.SendMailService"
 *		SCOPE="REQUEST">
 *		&lt;CONFIG
 *			mail_smtp_host="mail.nospamm.com"
 *			mail_from="XY@nospamm.com"
 *			mail_reply_to="XY@nospamm.com" >
 *			<MAIL_HEADERS
 *				X_mailer="Spago test Mailer" />
 *		&lt;/CONFIG>
 *	&lt;/SERVICE>
 * </pre>
 * 
 * @author buso
 *
 */
public class SendMailService extends AbstractService {
	/**
	 * 
	 */
	private static final long serialVersionUID = -765455164209311355L;
	private SourceBean _config = null;
	private Properties _sezProp = null;

	/**
	 * Possible configurations:
	 * <ul>
	 * <li>mail_smtp_host: name of the smtp server host</li>
	 * <li>mail_from: email for the from attribute of the email</li>
	 * <li>mail_reply_to: A comma separated list of recipient</li>
	 * <li>MAIL_HEADERS: a list of attributes to be inserted as headers</li>
	 * </ul>
	 */
	public void init(SourceBean config) throws Exception {
		// config: sender, smtpserver data
		_config = config;
		_sezProp = new Properties();
//		_sezProp.setProperty("mail.store.protocol", value);
//		_sezProp.setProperty("mail.transport.protocol", value);
//		_sezProp.setProperty("mail.host", value);
//		_sezProp.setProperty("mail.user", value);
//		_sezProp.setProperty("mail.from", value);
		_sezProp.setProperty("mail.smtp.host",
			(String)_config.getAttribute("mail_smtp_host"));
	}

	/**
	 * Create an email and send it.
	 * 
	 * @param to. A comma separated list of recipient.
	 * @param cc. A comma separated list of recipient.
	 * @param bcc. A comma separated list of recipient.
	 * @param subject. The subject for the email.
	 * @param text. The content of the email
	 * @param files. A comma separeted list of file names to attach to
	 * the email
	 * @throws Exception
	 */
	public void send(String to, String cc, String bcc,
		String subject, String text, String files) throws Exception {
		
		Session sez = Session.getInstance(_sezProp);
		MimeMessage msg = new MimeMessage(sez);
		Address from = new InternetAddress((String)_config
			.getAttribute("mail_from"));
		msg.setFrom(from);
		Object replyToStr = _config.getAttribute("mail_reply_to");
		if (replyToStr != null) {
			Address[] replyTo = InternetAddress.parse((String)replyToStr);
			msg.setReplyTo(replyTo);
		}

		if (!GenericValidator.isBlankOrNull(to)) {
			msg.setRecipients(RecipientType.TO, InternetAddress.parse(to, false));
		}
		if (!GenericValidator.isBlankOrNull(cc)) {
			msg.setRecipients(RecipientType.CC, InternetAddress.parse(cc, false));
		}
		if (!GenericValidator.isBlankOrNull(bcc)) {
			msg.setRecipients(RecipientType.BCC, InternetAddress.parse(bcc, false));
		}
		
		msg.setSubject(subject);
		
		// create multiparts if there are attachments
		if (!GenericValidator.isBlankOrNull(files)) {
			MimeMultipart parts = new MimeMultipart();
			MimeBodyPart textPart = new MimeBodyPart();
			textPart.setText(text);
			parts.addBodyPart(textPart);
			setAttachments(parts, files);
			msg.setContent(parts);
		} else {
			msg.setText(text);
		}

		setHeaders(msg);
	    msg.setSentDate(new Date());

	    Transport.send(msg);

	    TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
	    	"Mail message sent.");
	}

	private void setHeaders(MimeMessage msg) throws Exception {
		SourceBean headersConfig = (SourceBean)_config.getAttribute("MAIL_HEADERS");
		List headers = headersConfig.getContainedAttributes();
		for (Iterator iter = headers.iterator(); iter.hasNext();) {
			SourceBeanAttribute header = (SourceBeanAttribute)iter.next();
			msg.setHeader(header.getKey(), (String)header.getValue());
		}
	}
	
	/**
	 * Collect attachement rappresented by <code>files</code> in
	 * {@link Part} of the {@link MimeMessage}.
	 * 
	 * @param parts
	 * @param files
	 * @throws Exception
	 */
	private void setAttachments(MimeMultipart parts, String files) throws Exception {
		StringTokenizer strTkz = new StringTokenizer(files, ",");
		while (strTkz.hasMoreTokens()) {
			String fileName = strTkz.nextToken();
			File file = new File(fileName);
			if (file.exists() && file.isFile()) {
				FileDataSource fds = new FileDataSource(file);
				MimeBodyPart attachPart = new MimeBodyPart();
				attachPart.setDataHandler(new DataHandler(fds));
				attachPart.setFileName(fds.getName());
				parts.addBodyPart(attachPart);
			}
		}
	}
}
