/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.statistic;

import it.eng.spago.base.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.sql.Connection;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.apache.commons.validator.GenericValidator;
import org.apache.log4j.jdbc.JDBCAppender;
import org.apache.log4j.spi.LoggingEvent;

import com.jamonapi.Monitor;

/**
 * Log4j {@link JDBCAppender} implementation that use the
 * <code>LoggingEvent message</code> to write Jamon statistics to the
 * configured db.
 * Log4j configuration example
 * 
 * <pre>
 *  &lt;appender name="JamonJDBCAppender" class="it.eng.spago.statistic.JamonStatsJDBCAppender">
 *    &lt;param name="URL" value="jdbc:postgresql://localhost:5432/DeutscheBankStats" /> 
 *    &lt;param name="Driver" value="org.postgresql.Driver" /> 
 *    &lt;param name="User" value="DB" /> 
 *    &lt;param name="Password" value="DB" /> 
 *  &lt;/appender>
 * </pre>
 * <br/>
 * The {@link JDBCAppender#getConnection()} method is overrided to permit also
 * connection using datasources. The Log4j configuration change:
 * 
 * <pre>
 *  &lt;appender name="JamonJDBCAppender" class="it.eng.spago.statistic.JamonStatsJDBCAppender">
 *    &lt;param name="jndiName" value="java:comp/env/jdbc/hsql-spagodemo" /> 
 *  &lt;/appender>
 * </pre>
 * 
 * @author buso
 *
 */
public class JamonStatsJDBCAppender extends JDBCAppender {
	private String jndiName = "";
	
	protected String getLogStatement(LoggingEvent event) {
		Object message = event.getMessage();
		if (message == null && !(message instanceof String)) {
			return super.getLogStatement(event);
		}
		Timestamp now = new Timestamp(System.currentTimeMillis());
		// start creating insert sql statement
		StringBuffer statement = new StringBuffer()
			.append("INSERT INTO jamon_monitor ("
			+"timestamp,label,hits,avg,total,"
			+"units,stdDev,lastValue,min,max,"
			+"active,avgActive,maxActive,firstAccess,lastAccess"
			+") "
			+"VALUES ('"+now.toString()+"',");

		// add monitor values to the insert
		setValues(message, statement);
		// close the statement
		statement.append(")");
//		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
//			"JamonStatsJDBCAppender::getLogStatement: statement: "
//			+statement.toString());
		return statement.toString();
	}

	private void setValues(Object msg, StringBuffer stat) {
		Monitor mon = MonitorStringSerializer.deserialize((String)msg);
		stat.append("'").append(mon.getLabel()).append("',");
		stat.append(mon.getHits()).append(",");
		stat.append(mon.getAvg()).append(",");
		stat.append(mon.getTotal()).append(",");
		stat.append("'").append(mon.getUnits()).append("',");
		stat.append(mon.getStdDev()).append(",");
		stat.append(mon.getLastValue()).append(",");
		stat.append(mon.getMin()).append(",");
		stat.append(mon.getMax()).append(",");
		stat.append(mon.getActive()).append(",");
		stat.append(mon.getAvg()).append(",");
		stat.append(mon.getMaxActive()).append(",");
		Date firstAccess = new Date(mon.getFirstAccess().getTime());
		stat.append("'").append(firstAccess.toString()).append("',");
		Date lastAccess = new Date(mon.getLastAccess().getTime());
		stat.append("'").append(lastAccess.toString()).append("'");
	}

	public String getJndiName() {
		return jndiName;
	}

	/**
	 * Override {@link JDBCAppender} method to permit to obtain the connection
	 * from a datasource.
	 * If the <code>jndiName</code> parameter is not set in configuration file the
	 * {@link JDBCAppender} method will be used.
	 *
	 */
	/*
	 * NOTE: the closeConnection() method is not overrided,
	 * the JDBCAppender one is enough
	 */
	protected Connection getConnection() throws SQLException {
		if (connection == null) {
			if (GenericValidator.isBlankOrNull(jndiName)) {
				return super.getConnection();
			}
			try {
		        Context ctx = new InitialContext();
		        DataSource ds = (DataSource)ctx.lookup(jndiName);
		        connection = ds.getConnection();
		    }
		    catch (NamingException ne) {
		        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
		            "JamonStatsJDBCAppender::getConnection:", ne);
		        throw new SQLException("Naming Exception");
		    }
		}
		return connection;
	}

	/**
	 * Set the name of the datasource to use instead to use direct connection
	 * 
	 * @param jndiName
	 */
	public void setJndiName(String jndiName) {
		this.jndiName = jndiName.trim();
	}
}
