/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/

package it.eng.spago.testing;

import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.soap.axis.client.AdapterAxisProxy;
import it.eng.spago.testing.validators.TestValidatorIFace;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.util.List;

import org.xml.sax.InputSource;

/**
 * La classe <code>TestSuite</code> permette di eseguire "suite" di test
 * generate dalla funzione "recorder" (presente nell'AdapterHTTP) e salvate su
 * appositi file XML <code>TestSuite</code>
 * <p>
 * 
 * @version 1.0, 01/01/2008
 * @author Gabriele Rigamonti
 * @see ListValidators
 */
public class TestSuite implements Serializable {
	private static final long serialVersionUID = 1L;

	private final static int EXITCODE_GENERIC_ERROR = -1;

	private final static int EXITCODE_NOPARAM = -2;

	private final static int EXITCODE_LEVEL_ERROR = -3;

	private final static int EXITCODE_VALIDATORERROR = -4;

	private int _level = 0;

	private String _fileConfigName, _suiteEndPoint, _suiteTestName;

	private SourceBean _suiteConfigurationBean;

	private SourceBean _previousResponseExecution;

	private static PrintStream _out = System.err;

	public TestSuite(String fileConfigName, int level) throws Exception {
		_fileConfigName = fileConfigName;
		_level = level;
	}

	public static void main(String[] args) {
		if (args.length != 2) {
			log("Uso TestSuite <NomeFileSuite.xml> <livelloEsecuzione>");
			System.exit(EXITCODE_NOPARAM);
		}
		try {
			int level = Integer.parseInt(args[1]);
			TestSuite aTestSuite = new TestSuite(args[0], level);
			aTestSuite.runSuite();
		} catch (NumberFormatException ex) {
			log("Parametro livello errato o non numerico");
			System.exit(EXITCODE_LEVEL_ERROR);
		} catch (Exception ex) {
			log("Errore in esecuzione suite " + args[0] + " livello " + args[1]);
			System.exit(EXITCODE_LEVEL_ERROR);
		}
	}

	/**
	 * Suite coordinator
	 * 
	 * @throws Exception
	 */
	public void runSuite() throws Exception {
		boolean globalTestPassed = true;
		boolean singleTestPassed = false;
		_suiteConfigurationBean = getSuiteConfiguration();
		_suiteTestName = (String) _suiteConfigurationBean.getAttribute("SUITE.NAME");
		_suiteEndPoint = (String) _suiteConfigurationBean.getAttribute("SUITE.ENDPOINT");
		String testResult = "";
		List<SourceBean> testList = _suiteConfigurationBean.getAttributeAsList("SUITE.UNIT_TEST");
		log("....................................................................................................");
		log("Test Suite : " + _suiteTestName +  "  " + " endPoint: " + _suiteEndPoint);
		log("....................................................................................................\n");
		for (SourceBean testBean : testList) {
			singleTestPassed = runTest(testBean);
			globalTestPassed = globalTestPassed & singleTestPassed;
			testResult = singleTestPassed ? " HA SUPERATO IL TEST" : " Ha FALLITO IL TEST";
			log(testBean.getAttribute("testID") + testResult);
		}
		testResult = globalTestPassed ? " TEST SUITE SUPERATO" : " TEST SUITE FALLITO";
		log("\n\n........................Risultato Finale Test suite.....................................................");
		log("La Suite: " + _suiteTestName + " endPoint: " + _suiteEndPoint);
		log("");

		log(testResult);
		log("....................................................................................................");
	}

	/**
	 * Single Test coordinator
	 * 
	 * @param testBean
	 * @return
	 * @throws Exception
	 */
	private boolean runTest(SourceBean testBean) throws Exception {
		int enableLevel = Integer.parseInt("" + testBean.getAttribute("enableLevel"));
		if (enableLevel < _level)
			return true; // Test saltato per livello
		log("\n" + testBean.getAttribute("testID")
				+ ".......................................................................................");
		log("Descrizione Test: " + testBean.getAttribute("descrizione"));
		// Make service request for calling
		StringBuffer serviceRequest = getServiceRequest(testBean);
		// Execute soap call action
		executeProxy(serviceRequest);
		// Validate latest execution
		return validateTest(testBean, _previousResponseExecution);
	}

	private boolean validateTest(SourceBean testBean, SourceBean inputBean) {
		boolean singleTestResult = false;
		boolean completeTestResult = true;
		String result = null;
		String testEnabled = null;
		if (hasSomeResponseErrors((SourceBean) inputBean.getAttribute("ERRORS")))
			return false;

		List<SourceBean> validationList = testBean.getAttributeAsList("TEST_CONDITIONS.TEST");
		for (SourceBean validationBean : validationList) {
			if (validationBean != null) {
				testEnabled = (String) validationBean.getAttribute("ENABLE");
				if (testEnabled != null && testEnabled.equalsIgnoreCase("true")) {
					String className = (String) validationBean.getAttribute("CLASS");
					try {
						Class<TestValidatorIFace> validatorClassType = (Class<TestValidatorIFace>) ClassLoader
								.getSystemClassLoader().loadClass(className);
						TestValidatorIFace validatorClass = validatorClassType.newInstance();
						// valido prima i parametri
						if (validatorClass.validateParameters(validationBean)) {
							singleTestResult = validatorClass.validator(validationBean, inputBean, testBean);

							result = singleTestResult ? "Test Superato" : "Test Fallito";
							log("Il Validatore : " + className + " ha ottenuto il seguente esito : " + result);
							completeTestResult = completeTestResult & singleTestResult;
						} else {
							// ci sono errori nei parametri del validatore
							throw new Exception();
						}
					} catch (Exception ex) {
						log("Errore durante la chiamata del validatore : " + className);
						ex.printStackTrace();
						System.exit(EXITCODE_VALIDATORERROR);
					}

				}

			}
		}
		return completeTestResult;
	}

	private boolean hasSomeResponseErrors(SourceBean errorsBean) {
		boolean ret = false;
		for (Object error : errorsBean.getContainedAttributes()) {
			if (error != null) {
				if (error instanceof SourceBeanAttribute) {
					ret = true;
					log("Errore in esecuzione del servizio") ;
					Object value = ((SourceBeanAttribute)error).getValue() ;
					if (value instanceof SourceBean) {
						List<SourceBeanAttribute> errorDescList = ((SourceBean)value).getContainedAttributes() ;
						for (SourceBeanAttribute errorDesc : errorDescList) {
							log(((SourceBeanAttribute) errorDesc).getKey() +" : " 
							    + ((SourceBeanAttribute) errorDesc).getValue()) ; 
						}
					} else {
						log(((SourceBeanAttribute) error).getKey() +" : " + value) ;
					}
				}
			}

		}
		return ret;
	}

	/**
	 * Make request String concerning input testBean
	 * 
	 * ex: <SERVICE_REQUEST
	 * ACTION_NAME="ActLookupOperazioniRicercaBancaDepositaria" BANCA="Desio" />
	 * 
	 * @param testBean
	 * @return
	 * @throws Exception
	 */
	private StringBuffer getServiceRequest(SourceBean testBean) throws Exception {
		List<SourceBean> parameterList = testBean.getAttributeAsList("PARAMETERS.PARAMETER");
		StringBuffer serviceRequest = new StringBuffer("<SERVICE_REQUEST ");
		String parValue, parName;
		for (SourceBean parameterBean : parameterList) {
			if (parameterBean != null) {
				parName = (String) parameterBean.getAttribute("NAME");
				parValue = getParameterValue(parameterBean);
				serviceRequest.append(" " + parName + "=\"" + parValue + "\"");
			}
		}
		serviceRequest.append("/>");

		return serviceRequest;
	}

	private void executeProxy(StringBuffer serviceRequest) throws Exception {
		AdapterAxisProxy proxy1 = new AdapterAxisProxy();
		proxy1.setEndpoint(_suiteEndPoint);
		String response = proxy1.service(serviceRequest.toString());
		ByteArrayInputStream bais = new ByteArrayInputStream(response.getBytes());
		_previousResponseExecution = SourceBean.fromXMLStream(new InputSource(bais));
		int a = 0;
	}

	/**
	 * Ritorna il valore di un parametro espresso come SourceBean
	 * 
	 * es1: <PARAMETER name="ACTION_NAME"
	 * value="ActLookupOperazioniRichercaBancaDepositaria" type="ABSOLUTE" />
	 * es2: <PARAMETER name="par2" value="valore" type="RELATIVE" /> es3:
	 * <PARAMETER name="par2" />
	 * 
	 * Se il bean non contiene value o type viene ritornato l'attributo NAME
	 * 
	 * @param parameterBean
	 * @return
	 */
	private String getParameterValue(SourceBean parameterBean) {
		String value = (String) parameterBean.getAttribute("NAME");

		String type = (String) parameterBean.getAttribute("TYPE");
		if (type != null && type.equalsIgnoreCase("RELATIVE") && _previousResponseExecution != null) {
			value = (String) _previousResponseExecution.getAttribute("" + parameterBean.getAttribute("VALUE"));
		} else if (type != null && parameterBean.containsAttribute("VALUE")) {
			value = (String) parameterBean.getAttribute("VALUE");
		}

		return value;
	}

	/**
	 * Recupera la configurazione con gli Unit Test.
	 * 
	 * @return
	 * @throws IOException
	 * @throws SourceBeanException
	 */
	public SourceBean getSuiteConfiguration() throws IOException, SourceBeanException {
		SourceBean result = null;
		InputStream resourcesStream = getInputStream();

		try {
			result = SourceBean.fromXMLStream(new InputSource(resourcesStream));
		} catch (Exception ex) {
			throw new SourceBeanException(ex.getMessage());
		} finally {
			if (resourcesStream != null) {
				try {
					resourcesStream.close();
				} catch (Exception ex) {
					throw new SourceBeanException(ex.getMessage());
				}
			}
		}
		return result;
	}

	/**
	 * Recupera l'input stream dove leggere la configurazione
	 * 
	 * @return
	 * @throws FileNotFoundException
	 */
	public InputStream getInputStream() throws FileNotFoundException {
		return (new FileInputStream(_fileConfigName));

	}

	private static void log(String message) {
		_out.println(message);
	}
}
