/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util;

import it.eng.spago.base.Constants;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.SQLCommand;
import it.eng.spago.dbaccess.sql.result.DataResult;
import it.eng.spago.dbaccess.sql.result.InformationDataResult;
import it.eng.spago.dbaccess.sql.result.ScrollableDataResult;
import it.eng.spago.dbaccess.sql.result.std.DefaultScrollableDataResult;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import java.sql.ResultSet;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * DATE CONTRIBUTOR/DEVELOPER NOTE 13-12-2004 Butano - SourceBean ora
 * restituisce ArrayList 13-12-2005 Boccalon/Grassi - Added utility methods to
 * execute queries giving the DataConnection instead of the connection pool, or
 * giving the parameter list instead of the automatic parameters retrieval
 *
 * 14-03-2007 Grassi : Refactoring width StatementProvider new support classes
 * 
 */

public abstract class QueryExecutor {
	public static final String CREATE = "INSERT";
	public static final String READ = "READ";
	public static final String UPDATE = "UPDATE";
	public static final String DELETE = "DELETE";

	/**
	 * Crea il comando da eseguire a seconda del tipo di richiesta
	 * 
	 * @param dataConnection
	 * @param query
	 * @param type
	 *            Il tipo di query da eseguire: CREATE, READ, UPDATE, DELETE
	 * @return
	 */
	public static SQLCommand createStatementSql(final DataConnection dataConnection, final String statement, final String type) {
		SQLCommand sqlCommand = null;
		if (type.equalsIgnoreCase("INSERT"))
			sqlCommand = dataConnection.createInsertCommand(statement);
		else if (type.equalsIgnoreCase("UPDATE"))
			sqlCommand = dataConnection.createUpdateCommand(statement);
		else if (type.equalsIgnoreCase("DELETE"))
			sqlCommand = dataConnection.createDeleteCommand(statement);
		else
			sqlCommand = dataConnection.createSelectCommand(statement);
		return sqlCommand;
	}

	/**
	 * Apre la connessione attraverso il pool
	 * 
	 * @param pool
	 * @return
	 * @throws EMFInternalError
	 */
	public static DataConnection openConnection(final String pool, final RequestContainer requestContainer) throws EMFInternalError {
		return DataConnectionManager.getInstance().getConnection(pool, requestContainer);
	}

	/**
	 * Esegue i comandi SQL SELECT, INSERT, DELETE, UPDATE. La connessione viene
	 * chiesta al connection pool.
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param pool
	 *            Il nome del pool del database
	 * @param query
	 *            Il SourceBean contenente la configurazione della query da
	 *            eseguire
	 * @param type
	 *            Il tipo di query da eseguire: CREATE, READ, UPDATE, DELETE
	 * @return
	 */
	public static Object executeQuery(final RequestContainer requestContainer, final ResponseContainer responseContainer,
			final String pool, final SourceBean query, final String type) {
		Object result = null;
		DataConnection dataConnection = null;
		
		// A.G. 06 Set 2007
		// Eccezione inserita per la gestione delle query di configurazione
		// Tipicamente utilizzata dalla select eseguite dalle comboBox
		if (query != null && query.containsAttribute("key")) {
			return (new ConfigRowHandler()).getConfigForQuery(query, requestContainer, responseContainer) ;
		}
		
		try {
			// Apre la connessione
			dataConnection = openConnection(pool, requestContainer);
			result = executeQuery(requestContainer, responseContainer, dataConnection, query, type);
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "QueryExecutor::executeQuery:", (Exception) ex);
			responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex.getNativeException()));
		} // catch (Exception ex) try
		finally {
			Utils.releaseResources(dataConnection, null, null);
		} // finally try
		return result;
	} // public static Object executeQuery(RequestContainer

	/**
	 * Esegue i comandi SQL SELECT, INSERT, DELETE, UPDATE. La connessione 
	 * passata come parametro (per le transazioni gestite manualmente)
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param dataConnection
	 *            La connessione verso il database
	 * @param query
	 *            Il SourceBean contenente la configurazione della query da
	 *            eseguire
	 * @param type
	 *            Il tipo di query da eseguire: CREATE, READ, UPDATE, DELETE
	 * @return
	 */
	public static Object executeQuery(final RequestContainer requestContainer, final ResponseContainer responseContainer,
			DataConnection dataConnection, final SourceBean query, final String type) {
		Object result = null;

		try {
            StatementProvider statementProvider = new StatementProvider(requestContainer, responseContainer, dataConnection, query) ;
            String statement = statementProvider.getStatement() ;
            // Il DAO non ritorna alcuna query da eseguire (Ha gi fatto tutto da solo)
            if (statement.equals("")) {
            	return statementProvider.getReturnCodeFromDAO() ;
            }
            ArrayList inputParameters = (ArrayList)statementProvider.getParameterList() ;
			result = executeStatement(dataConnection, statement, type, inputParameters);
			if (result instanceof Boolean && ((Boolean)result).equals(Boolean.FALSE)) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING, "QueryExecutor::executeQuery: Errore in esecuzione: ha ritornato false") ;
			}
		} // try
		catch (EMFInternalError ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "QueryExecutor::executeQuery:", (Exception) ex);
			responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex.getNativeException()));
		} // catch (Exception ex) try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "QueryExecutor::executeQuery:", (Exception) ex);
			responseContainer.getErrorHandler().addError(new EMFInternalError(EMFErrorSeverity.ERROR, ex));
		} // catch (Exception ex) try
		return result;
	} // public static Object executeQuery(RequestContainer

	/**
	 * Esegue statement SQL con parametri passati esplicitamente nel metodo
	 * 
	 * @param dataConnection
	 *            La connessione di database: non viene effettuata alcuna commit
	 *            o rollback
	 * @param type
	 *            Il tipo di query da eseguire: CREATE, READ, UPDATE, DELETE
	 * @param query
	 *            Il SourceBean contenente la configurazione della query da
	 *            eseguire
	 * @param parameters
	 *            I parametri da integrare nello statement
	 * @return
	 * @throws Exception
	 */
	public static Object executeQuery(DataConnection dataConnection, String type, SourceBean query, ArrayList parameters)
			throws Exception {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "QueryExecutor::executeQuery: invocato");
		Object result = null;
		String statement = SQLStatements.getStatement((String) query.getAttribute("STATEMENT"));
		try {
			ArrayList inputParameters = new ArrayList();
			if (parameters != null) {
				for (Iterator it = parameters.iterator(); it.hasNext();) {
					inputParameters.add(dataConnection.createDataField("", Types.VARCHAR, (String) it.next()));
				}
			}
			result = executeStatement(dataConnection, statement, type, inputParameters);
			return result;
		} // try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "QueryExecutor::executeQuery:", ex);
			throw ex;
		} // catch (Exception ex) try
	} // public static SourceBean executeQuery(DataConnection dataConnection,

	/**
	 * Metodo base per l'esecuzione di uno statement SQL
	 * @param dataConnection
	 * @param statement
	 * @param type
	 * @param inputParameters
	 * @return
	 * @throws Exception
	 */
	public static Object executeStatement(DataConnection dataConnection, final String statement, final String type,
			final ArrayList inputParameters) throws Exception {
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "QueryExecutor::executeQuery: invocato");
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		Object result = null;
		try {
			// Crea il comando da eseguire a seconda del tipo di richiesta
			sqlCommand = createStatementSql(dataConnection, statement, type);
			// TracerSingleton.log("DEFAULT_LOGGER",
			// TracerSingleton.INFORMATION, "QueryExecutor::executeQuery:" +
			// statement);
			if ((inputParameters != null) && (inputParameters.size() != 0)) {
				/*
				 * String parlog = "" ; DataField par ; for (int i=0 ; i <
				 * inputParameters.size() ; i++) { par =
				 * (DataField)inputParameters.get(i) ; parlog += "par" + i + "
				 * ("+ par.getTypeName()+")='" + par.getStringValue() + "', " ; }
				 * TracerSingleton.log("DEFAULT_LOGGER",
				 * TracerSingleton.INFORMATION, "QueryExecutor::executeQuery:" +
				 * parlog);
				 */
				dataResult = sqlCommand.execute(inputParameters);
			} else
				dataResult = sqlCommand.execute();
			if (type.equalsIgnoreCase("SELECT") || type.equals(READ)) {
				ScrollableDataResult scrollableDataResult = (ScrollableDataResult) dataResult.getDataObject();
				result = scrollableDataResult.getSourceBean();
			} // if (type.equalsIgnoreCase("SELECT"))
			else {
				InformationDataResult informationDataResult = (InformationDataResult) dataResult.getDataObject();
				result = new Boolean(informationDataResult.getAffectedRows() > 0);
			} // if (type.equalsIgnoreCase("SELECT")) else
			return result;
		} // try
		catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "QueryExecutor::executeQuery:", ex);
			throw ex;
		} // catch (Exception ex) try
		finally {
			Utils.releaseResources(null, sqlCommand, dataResult);
		}
	}

	/**
	 * Return a byte array of requested field
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param fieldName  column name on db
	 * @param poolName 
	 * @param query
	 * @return
	 */
	public static List<byte[]> getBlob(final RequestContainer requestContainer, final ResponseContainer responseContainer, String fieldName, String poolName, SourceBean query) {
		DataConnection dataConnection = null;
		SQLCommand sqlCommand = null;
		DataResult dataResult = null;
		List<byte[]> rows = null ;
		byte[] blob = null ;
		try {
			dataConnection = DataConnectionManager.getInstance().getConnection(poolName, requestContainer);
			StatementProvider statementProvider = new StatementProvider(requestContainer, responseContainer, dataConnection, query) ;
            String statement = statementProvider.getStatement() ;
            List parameterList = statementProvider.getParameterList() ;
			sqlCommand = QueryExecutor.createStatementSql(dataConnection, statement, "SELECT");
			if ((parameterList != null) && (parameterList.size() != 0)) {
				dataResult = sqlCommand.execute(parameterList);
			} else {
				dataResult = sqlCommand.execute();
			}
			ResultSet rs = ((DefaultScrollableDataResult)dataResult.getDataObject()).getResultSet() ;
			do {
				blob = rs.getBytes(fieldName) ;
				if (rows == null) rows=new ArrayList<byte[]>();
				rows.add(blob);
			}while (rs.next()) ;
		} catch (Exception ex) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL,
					"ActFileRender:select - Errore durante lettura file da database", ex);
			responseContainer.getErrorHandler().addError(new EMFUserError(EMFErrorSeverity.ERROR, "Errore durante lettura file da database")) ;
		} finally {
			Utils.releaseResources(dataConnection, sqlCommand, dataResult);
		}
		return rows ;
	} // public boolean select(SourceBean request, SourceBean response)

} // public abstract class QueryExecutor
