/**

 Copyright 2004, 2008 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.util;

import it.eng.spago.base.Constants;
import it.eng.spago.base.NullValue;
import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.dao.IFaceQueryProvider;
import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.dbaccess.sql.DataField;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.paginator.smart.impl.DBRowProvider;
import it.eng.spago.tracing.TracerSingleton;
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
 * 
 * Classe <code>StatementProvider</code> responsabile della composizione degli statement SQL
 * Incorpora tutta la logica per la lettura delle configurazioni presenti nelle ACTION e nei MODULI
 * <p> 
 * @version 1.0, 14/03/2007
 * @author A. Grassi
 *
 */
public class StatementProvider implements Serializable {
	private static final long serialVersionUID = -3040846854013686164L;
	private DefaultRequestContext _requestContext ;
	private String _statement = null;
	private List _parameterList = null;
	private SourceBean _configQuery ;
    private DataConnection _dataConnection ;
    private Object _returnCodeFromDAO ;
    private String _sqlToAdd, _parameterStringValue ;
	
	/**
	 * Costruttore utilizzato tipicamente dalla DBRowProvider
	 * 
	 * @param requestContext
	 * @param dataConnection   connessione utilizzata per generare i parametri
	 * @param configQuery      rappresenta la radice nella configurazione "LIST_QUERY"
	 * @see DBRowProvider
	 */
	public StatementProvider(DefaultRequestContext requestContext, DataConnection dataConnection, SourceBean configQuery) {
		_requestContext = requestContext ;
		_configQuery = configQuery ;
		_dataConnection = dataConnection ;  
	}

	/**
	 * Costruttore dedicato al QueryExecutor
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param dataConnection   connessione utilizzata per generare i parametri
	 * @param configQuery      rappresenta la radice nella configurazione "SELECT_QUERY"
	 * @see DBRowProvider
	 */
	public StatementProvider(final RequestContainer requestContainer, final ResponseContainer responseContainer, DataConnection dataConnection, SourceBean configQuery) {
		_requestContext = new DefaultRequestContext(requestContainer, responseContainer) ;
		_configQuery = configQuery ;
		_dataConnection = dataConnection ;  
	}
	
	/**
	 * Ritorna lo statement in base alla configurazione inizializzata
	 * @return
	 * @throws Exception
	 */
	public String getStatement() throws Exception {
		makeStatement() ;
		return _statement ;
	}

	/**
	 * Ritorna la lista dei parametri recuperati per lo statement
	 * @return
	 * @throws Exception
	 */
	public List getParameterList() throws Exception {
		makeStatement() ;
		return _parameterList ;
	}

	/**
	 * Costruisce statement e relativi parametri e li salva nelle relative variabili di istanza
	 * @throws Exception
	 */
	private void makeStatement() throws Exception {
		if (_statement == null) {
			
			/*
			 * Se non trovo lo statement in configurazione, provo
			 * a ricercarlo nel "class provider"
			 */
			_statement = (String) _configQuery.getAttribute("STATEMENT");
			if (_statement == null || _statement.trim().length() == 0) {
				_statement = getStatementFromClassProvider() ;
			} else {
				String sqlStatement = SQLStatements.getStatement(_statement);
                if (sqlStatement != null && sqlStatement.trim().length() > 0)
                	_statement = sqlStatement ;
			}
			
			if (_statement != null && _statement.trim().length() > 0) {
				handleParameter() ;
				handleFilterParameter() ;
				handleGroupBy() ;
				handleOrderBy() ;
			}
		}
	}

	/**
	 * Statement "dinamici" generati da apposite classi
	 * 
	 * @param requestContext
	 * @param configQuery
	 * @return
	 * @throws Exception
	 */
	private String getStatementFromClassProvider() throws Exception {
        String className = (String)_configQuery.getAttribute("CLASS");
        if ((className == null) || className.equals("")) {
            TracerSingleton.log(
                Constants.NOME_MODULO,
                TracerSingleton.WARNING,
                "StatementProvider::getStatementFromClassProvider: statement non censito");
            throw new Exception("StatementProvider::getStatementFromClassProvider: statement non censito") ;
        } else { 
	        IFaceQueryProvider queryProvider = null;
	        try {
	            queryProvider = (IFaceQueryProvider) Class.forName(className).newInstance();
	        } // try
	        catch (Exception ex) {
	            TracerSingleton.log(
	                Constants.NOME_MODULO,
	                TracerSingleton.CRITICAL,
	                "DBRowProvider::open: classe query provider non valida",
	                ex);
	            throw ex ;
	        } // catch (Exception ex) try 
	        String query = queryProvider.getQuery(this);
	        _returnCodeFromDAO = queryProvider.getReturnCode() ;
	        List addParameter = queryProvider.getParameterList() ;
	        if (addParameter != null) {
	        	Iterator it = addParameter.iterator() ;
	        	while (it != null && it.hasNext()) {
	        		addParameter((DataField)it.next()) ;
	        	}
	        }
	        return query ;
        }
	}

	/**
	 * Crea la lista dei parametri ricercandone i valori sulla base delle informazioni
	 * impostate nella configurazione
	 * 
	 * @throws Exception
	 */
	private void handleParameter() throws Exception {
        List parameters = _configQuery.getAttributeAsList("PARAMETER");
        for (int i = 0; i < parameters.size(); i++) {
        	handleParameter((SourceBean) parameters.get(i), false) ;
        }
	}

	/**
	 * Crea la lista dei parametri di tipo filter (quelli che vengono inseriti
	 * solo se valorizzati). Aggiunge l'eventuale sql necessario
	 * 
	 * @throws Exception
	 */
	private void handleFilterParameter() throws Exception {
        List filterParameters = _configQuery.getAttributeAsList("FILTER_PARAMETER");
        for (int i = 0; i < filterParameters.size(); i++) {
        	handleParameter((SourceBean) filterParameters.get(i), true) ;
        }
	}
	
	/**
	 * Aggiunge le clausole di "GROUP BY"
	 * @throws Exception
	 */
	private void handleGroupBy() throws Exception {
        // Elabora tutte le sezioni di tipo "GROUP_BY"
        SourceBean groupByParameters = (SourceBean) _configQuery.getAttribute("GROUP_BY");
        if (groupByParameters != null) {
            addGroupByClause(groupByParameters) ;
        }
		
	}
	
	/**
	 * Aggiunge le clausole di "ORDER BY" e quelle di "SORT ACTION" (Ordinamento sulle colonne delle mappe di lista)
	 * @throws Exception
	 */
	private void handleOrderBy() throws Exception {
    	SourceBean orderByParameters = null;
        String sortAction =(String)_requestContext.getRequestContainer().getServiceRequest().getAttribute("SORT_ACTION") ;
        if (sortAction != null) { // Richiesto sort dinamico
        	addOrderByClause(sortAction) ;
        } else { // Elabora tutte le sezioni di tipo "ORDER_BY"
        	orderByParameters = (SourceBean) _configQuery.getAttribute("ORDER_BY");
            if (orderByParameters != null) {
            	addOrderByClause(orderByParameters) ;
            }
        }
	}

	/**
	 * Gestione generalizzata per i parametri di tipo "normale" e FILTER
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @param parameter
	 * @param inputParameters
	 * @param dataConnection
	 * @param isFilterParameter
	 * @param statement
	 * @param condizioneSql
	 * @return
	 */
	protected boolean handleParameter(final SourceBean parameter, boolean isFilterParameter) throws Exception {
	    boolean parameterUsed = false;
	    
	    // Lettura attributi di "tipo" del parametro
	    String parameterType = (String) parameter.getAttribute("TYPE");
	    String parameterValue = (String) parameter.getAttribute("VALUE");
	    String parameterScope = (String) parameter.getAttribute("SCOPE");
	    boolean uppercase = Boolean.parseBoolean((String)parameter.getAttribute("UPPERCASE", "FALSE"));

	    int dataType = getDataType(parameter) ;

        Object parameterValueObject = null ; 
        if (parameterType.equalsIgnoreCase("ABSOLUTE")) {
        	parameterValueObject = parameterValue;
        } else {
            parameterValueObject = 
            	ContextScooping.getScopedParameter(_requestContext.getRequestContainer(),
									_requestContext.getResponseContainer(),
									parameterValue, parameterScope, parameter);
        }
        
        // Aggiunta per Spago 2.0
        if (parameterValueObject instanceof NullValue) {
        	parameterValueObject = null ;
        }
        
        if (uppercase && parameterValueObject instanceof String) {
        	parameterValueObject = ((String)parameterValueObject).toUpperCase() ;
		}
        
	    if (!isFilterParameter) {
	    	/* In caso di parametro normale
	    	 * questo va aggiunto sempre indipendentemente dal fatto che sia o meno
	    	 * stato passato.
	    	 */
	    	addParameter(dataType, parameterValueObject) ;
	        parameterUsed = true;
	    } else {
	    	/* Per i parametri di tipo FILTER, bisogna
	    	 * prima valutare che effettivamente sia stato valorizzato
	    	 * prima di poterlo inserire nella lista dei parametri e di
	    	 * aggiungere il relativo codice sql 
	    	 */
	        if (parameterValueObject != null && parameterValueObject.toString().trim().length() > 0) {
	            _sqlToAdd = (String)parameter.getAttribute("SQL");
	            _parameterStringValue = parameterValueObject.toString() ;
	            makeFilterParameter(dataType, parameter, parameterValueObject) ;
	            parameterUsed = true;
	        } // if !inParameterValue
	    } // else !isFilterParameter
	    return parameterUsed;
	} // elaboraParametro
	
	
	/**
	 * Metodo per la generazione dei <FILTER_PARAMETER />
	 * Es di configurazione:
	 *    <FILTER_PARAMETER scope="SERVICE_REQUEST" type="RELATIVE" 
	 *       value="FILTER_tipodatoDesc"
	 *       uppercase="TRUE" 
	 *       sql="UPPER(et.APLONM || ' - ' || td.ATOLTT) like ? and bv.APSMNZ LIKE 'CICCIO' and cv.ABCDEF like ?" />
	 *       
	 * NB: 1) Parametro uppercase
	 *     2) sql contenente veri e propri statement
	 *     3) possono essere richieste pi sostituzioni dei parametri
	 *     
	 * I parameter con prefisso "FILTER_..." possono avere associato un ulteriore parametro "SELECT_..."
	 * contenete i criteri per la selezione.
	 * Valori ammessi per i criteri di selezione sono:
	 *        Minore di                suffisso "lt"
	 *        Minore o uguale a        suffisso "le"
	 *        Uguale a                 suffisso "eq"
	 *        Maggiore o uguale a      suffisso "ge"
	 *        Maggiore di              suffisso "gt"
	 *        Diverso da               suffisso "ne"
	 *        Contiene                 suffisso "like"
	 *        Comincia per             suffisso "startwith"
	 *        Non contiene             suffisso "notlike"
	 *        
	 * @param dataType
	 * @param sqlToAdd
	 * @param stringParameterValue
	 * @param parameterValue
	 * @throws Exception
	 */
	private void makeFilterParameter(int dataType, SourceBean filterParameter, Object parameterValue) throws Exception {
		handleSqlForSearchCriteria(filterParameter) ;
		// Aggiungo il parametro tante volte quanti sono i caratteri
		// '?' presenti nella stringa
        int countParams = StringUtils.count(_sqlToAdd, '?');
		for (int i = 0; countParams != 0 && i < countParams; i++) {
			if (! parameterValue.toString().equals(_parameterStringValue))
				addParameter(dataType, _parameterStringValue) ;
			else
				addParameter(dataType, parameterValue) ;
		}

        addWhereClause(_sqlToAdd);
	}
	
	/**
	 * Gestisce gli eventuali operatori associati ai parametri "FILTER_"
	 * @return true se deve essere utilizzato il parametro String false per l'object
	 */
	private void handleSqlForSearchCriteria(SourceBean filterParameter) {
		if (_sqlToAdd == null || _parameterStringValue == null)
			return  ;
		
		// <FILTER_PARAMETER scope="SERVICE_REQUEST" sql="UPPER(T1.ASO7TT) like ?" type="RELATIVE" uppercase="true" value="TYPED_SERVICE_REQUEST.FILTER_nomeFile"/>
		String parameterValue = (String)filterParameter.getAttribute("value") ;
		String searchCriteria =null ;
		int i ;
		if ((i=parameterValue.indexOf("FILTER_")) >= 0) {
			searchCriteria = (String)_requestContext.getRequestContainer().getServiceRequest().getAttribute("SELECT_"+parameterValue.substring(i+7)) ;
		}
		if (searchCriteria != null) {
			replaceSqlOperator(getSqlOp(searchCriteria)) ;
			
			if (searchCriteria.equalsIgnoreCase("like") ||
				searchCriteria.equalsIgnoreCase("notlike") ||
				searchCriteria.equalsIgnoreCase("none")) {
				boolean isDouble = true;
				try {
					double valore = Double.parseDouble(_parameterStringValue) ;
					isDouble = true;
					if (_parameterStringValue.endsWith(".0")) {
						_parameterStringValue = _parameterStringValue.substring(0, _parameterStringValue.length()-2) ;
					}
				} catch (Exception ex) {
					isDouble = false;
				}
				_parameterStringValue =  "%"+_parameterStringValue+"%" ;
			}
			else if (searchCriteria.equalsIgnoreCase("startwith")) {
				_parameterStringValue =  _parameterStringValue+"%" ;
			}
		}
		else {
			if (_sqlToAdd.toUpperCase().indexOf("LIKE")>=0)
				_parameterStringValue =  "%"+_parameterStringValue+"%" ;
		}
	}
	
	/**
	 * Mappa il searchCriteria con l'operatore SQL
	 *        Minore di                suffisso "lt"
	 *        Minore o uguale a        suffisso "le"
	 *        Uguale a                 suffisso "eq"
	 *        Maggiore o uguale a      suffisso "ge"
	 *        Maggiore di              suffisso "gt"
	 *        Diverso da               suffisso "ne"
	 *        Contiene                 suffisso "like"
	 *        Comincia per             suffisso "startwith"
	 *        Non contiene             suffisso "notlike"
	 * 
	 * @param searchCriteria
	 * @return
	 */
	public static String getSqlOp(String searchCriteria) {
		if (searchCriteria == null || searchCriteria.trim().length() == 0 || searchCriteria.trim().equalsIgnoreCase("NONE"))
			return "like" ;
		else if (searchCriteria.equalsIgnoreCase("lt"))
			return "<" ;
		else if (searchCriteria.equalsIgnoreCase("le"))
			return "<=" ;
		else if (searchCriteria.equalsIgnoreCase("eq"))
			return "=" ;
		else if (searchCriteria.equalsIgnoreCase("ge"))
			return ">=" ;
		else if (searchCriteria.equalsIgnoreCase("gt"))
			return ">" ;
		else if (searchCriteria.equalsIgnoreCase("ne"))
			return "<>" ;
		else if (searchCriteria.equalsIgnoreCase("like") ||
				searchCriteria.equalsIgnoreCase("startwith"))
			return "LIKE" ;
		else if (searchCriteria.equalsIgnoreCase("notlike"))
			return "NOT LIKE" ;

		return "LIKE" ;
	}
	
	/**
	 * Sostituisce nella variabile d'istanza _sqlToAdd l'operatore richiesto
	 * @param replaceOp
	 */
	private void replaceSqlOperator(String replaceOp) {
		
		// Verifica se esiste almeno un parametro da sostituire altrimenti ritorna al chiamante
		if (_sqlToAdd.lastIndexOf("?")<0)
			return;
		
		String[] items = _sqlToAdd.split("\\?");
		String sqlMod ="";
		for(String element : items) {
			if (element.toUpperCase().lastIndexOf("LIKE")>=0) 
				element = element.toUpperCase().replace("LIKE", replaceOp);
			else if (element.lastIndexOf(">=")>=0) 
				element = element.replace(">=", replaceOp);
			else if (element.lastIndexOf("<=")>=0) 
				element = element.replace("<=", replaceOp);
			else if (element.lastIndexOf("<>")>=0) { 
				element = element.replace("<>", replaceOp);
			} else if (element.lastIndexOf(">")>=0) { 
					element = element.replace(">", replaceOp);
			} else if (element.lastIndexOf("<")>=0)  
						element = element.replace("<", replaceOp);
			else if (element.lastIndexOf("=")>=0) 
				element = element.replace("=", replaceOp);
			sqlMod += element + " ? ";
		}
		_sqlToAdd = sqlMod ;
		
		/*
		StringTokenizer str = new StringTokenizer(_sqlToAdd, "?") ;
		
		if (_sqlToAdd != null && _sqlToAdd.toUpperCase().indexOf("CONVERT(") != -1)
			return ; // Bypass Sybase convert function
		
		while (str.hasMoreTokens()) {
			sqlMod = str.nextToken().trim() ;
			// " ...  LIKE ?" oppure " ... >= ?"  NON Effettua la sostituzione se diverso da LIKE
			if (sqlMod.toUpperCase().endsWith("LIKE")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf("LIKE")-1) + " " + replaceOp ;
			} else if (sqlMod.toUpperCase().endsWith(">=")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf(">=")-1) + " " + replaceOp ;
			} else if (sqlMod.toUpperCase().endsWith("<=")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf("<=")-1) + " " + replaceOp ;
			} else if (sqlMod.toUpperCase().endsWith(">")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf(">")-1) + " " + replaceOp ;
			} else if (sqlMod.toUpperCase().endsWith("<")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf("<")-1) + " " + replaceOp ;
			} else if (sqlMod.toUpperCase().endsWith("=")) {
				sqlMod = sqlMod.substring(0, sqlMod.toUpperCase().lastIndexOf("=")-1) + " " + replaceOp ;
			}
			sqlMod += " ?" ;
		}
		_sqlToAdd = sqlMod ;
		*/
	}
	
	/**
	 * Aggiunge un parametro alla lista
	 * @param dataType
	 * @param parameterValueObject
	 * @throws Exception
	 */
	private void addParameter(int dataType, Object parameterValueObject) throws Exception {
        if (_parameterList == null)
        	_parameterList = new ArrayList();

       	_parameterList.add(getDataField(dataType, parameterValueObject ));
	}
	
	/**
	 * Aggiunge un parametro alla lista
	 * @param DataField
	 * @throws Exception
	 */
	private void addParameter(DataField df) throws Exception {
        if (_parameterList == null)
        	_parameterList = new ArrayList();

       	_parameterList.add(df);
	}
	
	/**
	 * Costruisce il DataField per i parametri passati
	 * @param dataType
	 * @param parameterValueObject
	 * @return
	 */
	public DataField getDataField(int dataType, Object parameterValueObject) {
		int type = dataType ;
		Object value = parameterValueObject ;
		if (!_dataConnection.isAllowNullValues()) {
			
			//if (dataType == Types.VARCHAR || parameterValueObject == null) {
	        //	if (parameterValueObject != null)
	        //		value = parameterValueObject.toString() ;
	        	//else
	        	//	value = "" ;
	        	//type = Types.VARCHAR ;
			//}
		
			if (parameterValueObject==null) {
	            switch (dataType) {
	                case Types.CHAR:
	                case Types.VARCHAR:
	                case Types.LONGVARCHAR:
	                	value = "";
	                    break;
	                case Types.BIGINT:
	                case Types.BIT:
	                case Types.DECIMAL:
	                case Types.DOUBLE:
	                case Types.FLOAT:
	                case Types.INTEGER:
	                case Types.NUMERIC:
	                case Types.REAL:
	                case Types.SMALLINT:
	                case Types.TINYINT:
	                	value = new BigDecimal(0);
	                    break;
	                case Types.BINARY:
	                case Types.LONGVARBINARY:
	                case Types.VARBINARY:
	                    break;
	                case Types.DATE:
	                    break;
	                case Types.TIMESTAMP:
	                    break;
	                default:
	            } // switch (sqlType)
			}
		}
    	return _dataConnection.createDataField("", type, value) ;
	}
	
	/**
	 * Aggiunge condizioni allo statement
	 * @param sqlToAdd
	 */
	private void addWhereClause(String sqlToAdd) {
        String condizioneSql = (_statement.toUpperCase().indexOf("WHERE") != -1) ? " AND " : " WHERE ";
        _statement+=condizioneSql ;
        _statement+= sqlToAdd;
	}
	
	/**
	 * Aggiunge raggruppamenti allo statement
	 * @param sqlToAdd
	 */
	private void addGroupByClause(SourceBean sqlBean) {
		String sqlToAdd =(String)sqlBean.getAttribute("SQL") ;
		if (sqlToAdd != null && sqlToAdd.trim().length() > 0) {
			_statement+= " GROUP BY " + sqlToAdd;
		}
	}

	/**
	 * Aggiunge ordinamenti allo statement
	 * @param sqlToAdd
	 */
	private void addOrderByClause(SourceBean sqlBean) {
		String sqlToAdd =(String)sqlBean.getAttribute("SQL") ;
		if (sqlToAdd != null && sqlToAdd.trim().length() > 0) {
	        String condizioneSql = (_statement.indexOf("ORDER BY") != -1) ? "," : " ORDER BY ";
	         _statement+= condizioneSql + sqlToAdd ;
		}
	}

	/**
	 * Aggiunge ordinamenti allo statement
	 * @param sqlToAdd
	 */
	private void addOrderByClause(String sqlToAdd) {
		if (sqlToAdd != null && sqlToAdd.trim().length() > 0) {
	        String condizioneSql = (_statement.indexOf("ORDER BY") != -1) ? "," : " ORDER BY ";
	        
	        StringTokenizer orderClauses = new StringTokenizer(sqlToAdd,",");
	        
	        String orderClause;
	        StringTokenizer parser;
	        String parsed;
	        
	        String sortParameters = new String();
	        while (orderClauses.hasMoreTokens()) {
	        	orderClause = orderClauses.nextToken();
	        	parser = new StringTokenizer(orderClause);
	        	parsed = parser.nextToken();
	        	if (_configQuery.getFilteredSourceBeanAttribute("SORT_ACTION", "SQL", parsed)!=null) {
	        		sortParameters+=orderClause;
		        	if (orderClauses.hasMoreTokens()) {
		        		sortParameters+=",";
		        	}
	        	}
	        	

			}
	       
	        if (sortParameters.length()>0) {
	        	_statement+= condizioneSql+sortParameters;
	        }
	        
	       
		}
	}

	/**
	 * Se non specificato il datatype utilizzato per la creazione dei parametri  il VARCHAR
	 * Questo metodo mappa il tipo scelto con il relativo SQL
	 * @param parameter
	 * @return
	 */
	private int getDataType(SourceBean parameter) {
	    String dataTypeName = (String) parameter.getAttribute("DATATYPE");
    	int dataType = Types.VARCHAR ;
	    if (dataTypeName != null) {
	    	dataTypeName = dataTypeName.trim().toUpperCase() ;
	    	if (dataTypeName.equals("ARRAY"))
	    		dataType = Types.ARRAY ;
	    	else if (dataTypeName.equals("BIGINT"))
    			dataType = Types.BIGINT ;
	    	else if (dataTypeName.equals("BINARY"))
    			dataType = Types.BINARY ;
	    	else if (dataTypeName.equals("BIT"))
    			dataType = Types.BIT ;
	    	else if (dataTypeName.equals("TIMESTAMP"))
    			dataType = Types.TIMESTAMP ;
	    	else if (dataTypeName.equals("DATE"))
    			dataType = Types.DATE ;
	    	else if (dataTypeName.equals("BLOB"))
    			dataType = Types.BLOB ;
	    	else if (dataTypeName.equals("DECIMAL"))
    			dataType = Types.DECIMAL ;
	    }
	    return dataType ;
    }

	public SourceBean getConfigQuery() {
		return _configQuery;
	}

	public DataConnection getDataConnection() {
		return _dataConnection;
	}

	public DefaultRequestContext getRequestContext() {
		return _requestContext;
	}

	public Object getReturnCodeFromDAO() {
		return _returnCodeFromDAO;
	}
}